@@ -4,32 +4,35 @@ import logging.config as logging_config
 import os
 from contextlib import contextmanager
 import numpy as np
-import shutil
+
+import mot
+from mot.configuration import CLRuntimeInfo, CLRuntimeAction
 from .__version__ import VERSION, VERSION_STATUS, __version__
 
-from mdt.configuration import get_logging_configuration_dict
+from mdt.configuration import get_logging_configuration_dict, get_optimizer_for_model, get_general_sampling_settings
+
 try:
     logging_config.dictConfig(get_logging_configuration_dict())
 except ValueError:
     print('Logging disabled')
 
 from mdt.component_templates.parameters import FreeParameterTemplate, ProtocolParameterTemplate
-from mdt.component_templates.cascade_models import CascadeTemplate
 from mdt.component_templates.batch_profiles import BatchProfileTemplate
 from mdt.component_templates.compartment_models import CompartmentTemplate, WeightCompartmentTemplate
 from mdt.component_templates.composite_models import CompositeModelTemplate
 from mdt.component_templates.library_functions import LibraryFunctionTemplate
 
-from mdt.lib.model_fitting import get_batch_fitting_function
-from mdt.utils import estimate_noise_std, get_cl_devices, load_input_data,\
-    create_blank_mask, create_index_matrix, \
+from mdt.lib.processing.model_fitting import get_batch_fitting_function
+from mdt.utils import estimate_noise_std, get_cl_devices, create_blank_mask, create_index_matrix, \
     volume_index_to_roi_index, roi_index_to_volume_index, load_brain_mask, init_user_settings, restore_volumes, \
     apply_mask, create_roi, volume_merge, protocol_merge, create_median_otsu_brain_mask, create_brain_mask, \
     load_samples, load_sample, load_nifti, write_slice_roi, apply_mask_to_file, extract_volumes, \
     get_slice_in_dimension, per_model_logging_context, \
-    get_temporary_results_dir, get_example_data, SimpleInitializationData, InitializationData, load_volume_maps,\
-    covariance_to_correlation, check_user_components, unzip_nifti, zip_nifti
-from mdt.lib.sorting import sort_orientations, create_sort_matrix, sort_volumes_per_voxel
+    get_temporary_results_dir, get_example_data, SimpleInitializationData, InitializationData, load_volume_maps, \
+    covariance_to_correlation, check_user_components, unzip_nifti, zip_nifti, combine_dict_to_array, \
+    compute_noddi_dti
+from mdt.lib.input_data import load_input_data
+from mdt.lib.sorting import sort_orientations, create_4d_sort_matrix, sort_volumes_per_voxel
 from mdt.simulations import create_signal_estimates, simulate_signals, add_rician_noise
 from mdt.lib.batch_utils import run_function_on_batch_fit_output, batch_apply, \
     batch_profile_factory, get_subject_selection
@@ -37,7 +40,7 @@ from mdt.protocols import load_bvec_bval, load_protocol, auto_load_protocol, wri
     create_protocol
 from mdt.configuration import config_context, get_processing_strategy, get_config_option, set_config_option
 from mdt.lib.exceptions import InsufficientProtocolError
-from mdt.lib.nifti import write_nifti
+from mdt.lib.nifti import write_nifti, get_all_nifti_data
 from mdt.lib.components import get_model, get_batch_profile, get_component, get_template
 
 
@@ -45,20 +48,69 @@ __author__ = 'Robbert Harms'
 __date__ = "2015-03-10"
 __license__ = "LGPL v3"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
+
+
+def get_optimization_inits(model_name, input_data, output_folder, cl_device_ind=None,
+                           method=None, optimizer_options=None, double_precision=False):
+    """Get better optimization starting points for the given model.
+
+    Since initialization can make quite a difference in optimization results, this function can generate
+    a good initialization starting point for the given model. The idea is that before you call the :func:`fit_model`
+    function, you call this function to get a better starting point. An usage example would be::
+
+        input_data = mdt.load_input_data(..)
+
+        init_data = get_optimization_inits('BallStick_r1', input_data, '/my/folder')
+
+        fit_model('BallStick_r1', input_data, '/my/folder',
+                  initialization_data={'inits': init_data})
+
+    Where the init data returned by this function can directly be used as input to the ``initialization_data``
+    argument of the :func`fit_model` function.
+
+    Please note that his function only supports models shipped by default with MDT.
+
+    Args:
+        model_name (str):
+            The name of a model for which we want the optimization starting points.
+        input_data (:class:`~mdt.lib.input_data.MRIInputData`): the input data object containing all
+            the info needed for model fitting of intermediate models.
+        output_folder (string): The path to the folder where to place the output, we will make a subdir with the
+            model name in it.
+        cl_device_ind (int or list): the index of the CL device to use. The index is from the list from the function
+            utils.get_cl_devices(). This can also be a list of device indices.
+        method (str): The optimization method to use, one of:
+            - 'Levenberg-Marquardt'
+            - 'Nelder-Mead'
+            - 'Powell'
+            - 'Subplex'
+
+            If not given, defaults to 'Powell'.
+        optimizer_options (dict): extra options passed to the optimization routines.
+        double_precision (boolean): if we would like to do the calculations in double precision
+
+    Returns:
+        dict: a dictionary with initialization points for the selected model
+    """
+    from mdt.lib.processing.model_fitting import get_optimization_inits
+    return get_optimization_inits(model_name, input_data, output_folder, cl_device_ind=cl_device_ind,
+                                  method=method, optimizer_options=optimizer_options,
+                                  double_precision=double_precision)
 
 
 def fit_model(model, input_data, output_folder,
-              method=None, recalculate=False, only_recalculate_last=False,
-              cl_device_ind=None, double_precision=False, tmp_results_dir=True,
-              initialization_data=None, post_processing=None, optimizer_options=None):
+              method=None, optimizer_options=None, recalculate=False,
+              cl_device_ind=None, cl_load_balancer=None,
+              double_precision=False, tmp_results_dir=True,
+              initialization_data=None, use_cascaded_inits=True,
+              post_processing=None):
     """Run the optimizer on the given model.
 
     Args:
-        model (str or :class:`~mdt.models.composite.DMRICompositeModel` or :class:`~mdt.models.cascade.DMRICascadeModelInterface`):
-            An implementation of an AbstractModel that contains the model we want to optimize or the name of
-            an model.
-        input_data (:class:`~mdt.utils.MRIInputData`): the input data object containing all
+        model (str or :class:`~mdt.models.base.EstimableModel`):
+            The name of a composite model or an implementation of a composite model.
+        input_data (:class:`~mdt.lib.input_data.MRIInputData`): the input data object containing all
             the info needed for the model fitting.
         output_folder (string): The path to the folder where to place the output, we will make a subdir with the
             model name in it.
@@ -69,68 +121,103 @@ def fit_model(model, input_data, output_folder,
             - 'Subplex'
 
             If not given, defaults to 'Powell'.
-
+        optimizer_options (dict): extra options passed to the optimization routines.
         recalculate (boolean): If we want to recalculate the results if they are already present.
-        only_recalculate_last (boolean):
-            This is only of importance when dealing with CascadeModels.
-            If set to true we only recalculate the last element in the chain (if recalculate is set to True, that is).
-            If set to false, we recalculate everything. This only holds for the first level of the cascade.
-        cl_device_ind (int or list): the index of the CL device to use. The index is from the list from the function
-            utils.get_cl_devices(). This can also be a list of device indices.
+        cl_device_ind (List[Union[mot.lib.cl_environments.CLEnvironment, int]]
+                             or mot.lib.cl_environments.CLEnvironment or int): the CL devices to use.
+            Either provide MOT CLEnvironment's or indices from into the list from the function mdt.get_cl_devices().
+        cl_load_balancer (mot.lib.load_balancers.LoadBalancer or Tuple[float]): the load balancer to use. Can also
+            be an array of fractions (summing to 1) with one fraction per device. For example, for two devices one
+            can specify ``cl_load_balancer = [0.3, 0.7]`` to let one device to more work than another.
         double_precision (boolean): if we would like to do the calculations in double precision
         tmp_results_dir (str, True or None): The temporary dir for the calculations. Set to a string to use
             that path directly, set to True to use the config value, set to None to disable.
-        initialization_data (:class:`~mdt.utils.InitializationData` or dict): provides (extra) initialization data to
-            use during model fitting. If we are optimizing a cascade model this data only applies to the last model
-            in the cascade. If a dictionary is given we will load the elements as arguments to the
-            :class:`mdt.utils.SimpleInitializationData` class. For example::
+        initialization_data (dict): provides (extra) initialization data to
+            use during model fitting. This dictionary can contain the following elements:
 
-                initialization_data = {'fixes': {...}, 'inits': {...}}
+            * ``inits``: dictionary with per parameter an initialization point
+            * ``fixes``: dictionary with per parameter a fixed point, this will remove that parameter from the fitting
+            * ``lower_bounds``: dictionary with per parameter a lower bound
+            * ``upper_bounds``: dictionary with per parameter a upper bound
+            * ``unfix``: a list of parameters to unfix
 
-            is transformed into::
+            For example::
 
-                initialization_data = SimpleInitializationData(fixes={...}, inits={...})
+                initialization_data = {
+                    'fixes': {'Stick0.theta: np.array(...), ...},
+                    'inits': {...}
+                }
+
+        use_cascaded_inits (boolean): if set, we initialize the model parameters using :func:`get_optimization_inits`.
+            You can also overrule the default initializations using the ``initialization_data`` attribute.
         post_processing (dict): a dictionary with flags for post-processing options to enable or disable.
             For valid elements, please see the configuration file settings for ``optimization``
             under ``post_processing``. Valid input for this parameter is for example: {'covariance': False}
             to disable automatic calculation of the covariance from the Hessian.
-        optimizer_options (dict): extra options passed to the optimization routines.
 
     Returns:
         dict: The result maps for the given composite model or the last model in the cascade.
             This returns the results as 3d/4d volumes for every output map.
     """
-    import mdt.utils
-    from mdt.lib.model_fitting import ModelFit
+    initialization_data = initialization_data or {}
+    logger = logging.getLogger(__name__)
 
-    if not mdt.utils.check_user_components():
+    if not check_user_components():
         init_user_settings(pass_if_exists=True)
 
-    if not isinstance(initialization_data, InitializationData) and initialization_data is not None:
-        initialization_data = SimpleInitializationData(**initialization_data)
+    cl_runtime_info = CLRuntimeInfo(cl_environments=cl_device_ind,
+                                    double_precision=double_precision,
+                                    load_balancer=cl_load_balancer)
 
-    if cl_device_ind is not None and not isinstance(cl_device_ind, collections.Iterable):
-        cl_device_ind = [cl_device_ind]
+    if isinstance(model, str):
+        model_name = model
+        model_instance = get_model(model)()
+    else:
+        model_name = model.name
+        model_instance = model
 
-    model_fit = ModelFit(model, input_data, output_folder, method=method, optimizer_options=optimizer_options,
-                         recalculate=recalculate,
-                         only_recalculate_last=only_recalculate_last,
-                         cl_device_ind=cl_device_ind, double_precision=double_precision,
-                         tmp_results_dir=tmp_results_dir, initialization_data=initialization_data,
-                         post_processing=post_processing)
-    return model_fit.run()
+    if not model_instance.is_input_data_sufficient(input_data):
+        raise InsufficientProtocolError(
+            'The provided protocol is insufficient for this model. '
+            'The reported errors where: {}'.format(model_instance.get_input_data_problems(input_data)))
+
+    if post_processing:
+        model_instance.update_active_post_processing('optimization', post_processing)
+
+    if use_cascaded_inits:
+        initialization_data['inits'] = initialization_data.get('inits', {})
+        inits = get_optimization_inits(model_name, input_data, output_folder, cl_device_ind=cl_device_ind,
+                                       method=method, optimizer_options=optimizer_options,
+                                       double_precision=double_precision)
+        inits.update(initialization_data['inits'])
+        initialization_data['inits'] = inits
+        logger.info('Preparing {0} with the cascaded initializations.'.format(model_name))
+
+    initialization_data = SimpleInitializationData(**initialization_data)
+    initialization_data.apply_to_model(model_instance, input_data)
+
+    if method is None:
+        method, optimizer_options = get_optimizer_for_model(model_name)
+
+    with mot.configuration.config_context(CLRuntimeAction(cl_runtime_info)):
+        from mdt.lib.processing.model_fitting import fit_composite_model
+        fit_composite_model(model_instance, input_data, output_folder, method,
+                            get_temporary_results_dir(tmp_results_dir), recalculate=recalculate,
+                            optimizer_options=optimizer_options)
+
+    return get_all_nifti_data(os.path.join(output_folder, model_name))
 
 
 def sample_model(model, input_data, output_folder, nmr_samples=None, burnin=None, thinning=None,
-                 method=None, recalculate=False, cl_device_ind=None, double_precision=False,
+                 method=None, recalculate=False, cl_device_ind=None, cl_load_balancer=None, double_precision=False,
                  store_samples=True, sample_items_to_save=None, tmp_results_dir=True,
-                 initialization_data=None, post_processing=None
-                 ):
-    """Sample a composite model using the Adaptive Metropolis-Within-Gibbs (AMWG) MCMC algorithm [1].
+                 initialization_data=None, post_processing=None, post_sampling_cb=None,
+                 sampler_options=None):
+    """Sample a composite model using Markov Chain Monte Carlo sampling.
 
     Args:
-        model (:class:`~mdt.models.composite.DMRICompositeModel` or str): the model to sample
-        input_data (:class:`~mdt.utils.MRIInputData`): the input data object containing all
+        model (str or :class:`~mdt.models.base.EstimableModel`): the model to sample
+        input_data (:class:`~mdt.lib.input_data.MRIInputData`): the input data object containing all
             the info needed for the model fitting.
         output_folder (string): The path to the folder where to place the output, we will make a subdir with the
             model name in it (for the optimization results) and then a subdir with the samples output.
@@ -149,8 +236,12 @@ def sample_model(model, input_data, output_folder, nmr_samples=None, burnin=None
             If not given, defaults to 'AMWG'.
 
         recalculate (boolean): If we want to recalculate the results if they are already present.
-        cl_device_ind (int): the index of the CL device to use. The index is from the list from the function
-            utils.get_cl_devices().
+        cl_device_ind (List[Union[mot.lib.cl_environments.CLEnvironment, int]]
+                             or mot.lib.cl_environments.CLEnvironment or int): the CL devices to use.
+            Either provide MOT CLEnvironment's or indices from into the list from the function mdt.get_cl_devices().
+        cl_load_balancer (mot.lib.load_balancers.LoadBalancer or Tuple[float]): the load balancer to use. Can also
+            be an array of fractions (summing to 1) with one fraction per device. For example, for two devices one
+            can specify ``cl_load_balancer = [0.3, 0.7]`` to let one device to more work than another.
         double_precision (boolean): if we would like to do the calculations in double precision
         store_samples (boolean): determines if we store any of the samples. If set to False we will store none
             of the samples.
@@ -159,36 +250,47 @@ def sample_model(model, input_data, output_folder, nmr_samples=None, burnin=None
             items 'LogLikelihood' and 'LogPrior'.
         tmp_results_dir (str, True or None): The temporary dir for the calculations. Set to a string to use
                 that path directly, set to True to use the config value, set to None to disable.
-        initialization_data (:class:`~mdt.utils.InitializationData` or dict): provides (extra) initialization data to
-            use during model fitting. If we are optimizing a cascade model this data only applies to the last model
-            in the cascade. If a dictionary is given we will load the elements as arguments to the
-            :class:`mdt.utils.SimpleInitializationData` class. For example::
+        initialization_data (dict): provides (extra) initialization data to
+            use during model fitting. This dictionary can contain the following elements:
 
-                initialization_data = {'fixes': {...}, 'inits': {...}}
+            * ``inits``: dictionary with per parameter an initialization point
+            * ``fixes``: dictionary with per parameter a fixed point, this will remove that parameter from the fitting
+            * ``lower_bounds``: dictionary with per parameter a lower bound
+            * ``upper_bounds``: dictionary with per parameter a upper bound
+            * ``unfix``: a list of parameters to unfix
 
-            is transformed into::
+            For example::
+
+                initialization_data = {
+                    'fixes': {'Stick0.theta: np.array(...), ...},
+                    'inits': {...}
+                }
 
-                initialization_data = SimpleInitializationData(fixes={...}, inits={...})
         post_processing (dict): a dictionary with flags for post-processing options to enable or disable.
             For valid elements, please see the configuration file settings for ``sample`` under ``post_processing``.
-            Valid input for this parameter is for example: {'sample_statistics': True} to enable automatic calculation
-            of the sample statistics.
+            Valid input for this parameter is for example: {'univariate_normal': True} to enable automatic calculation
+            of the univariate normal distribution for the model parameters.
+        post_sampling_cb (Callable[
+            [mot.sample.base.SamplingOutput, mdt.models.composite.DMRICompositeModel], Optional[Dict]]):
+                additional post-processing called after sampling. This function can optionally return a (nested)
+                dictionary with as keys dir-/file-names and as values maps to be stored in the results directory.
+        sampler_options (dict): specific options for the MCMC routine. These will be provided to the sampling routine
+            as additional keyword arguments to the constructor.
 
     Returns:
         dict: if store_samples is True then we return the samples per parameter as a numpy memmap. If store_samples
             is False we return None
-
-    References:
-        1. Roberts GO, Rosenthal JS. Examples of adaptive MCMC. J Comput Graph Stat. 2009;18(2):349-367.
-           doi:10.1198/jcgs.2009.06134.
     """
-    import mdt.utils
-    from mot.lib.load_balance_strategies import EvenDistribution
-    from mdt.lib.model_sampling import sample_composite_model
-    from mdt.models.cascade import DMRICascadeModelInterface
-    import mot.configuration
+    initialization_data = initialization_data or {}
 
-    settings = mdt.configuration.get_general_sampling_settings()
+    if not check_user_components():
+        init_user_settings(pass_if_exists=True)
+
+    cl_runtime_info = CLRuntimeInfo(cl_environments=cl_device_ind,
+                                    double_precision=double_precision,
+                                    load_balancer=cl_load_balancer)
+
+    settings = get_general_sampling_settings()
     if nmr_samples is None:
         nmr_samples = settings['nmr_samples']
     if burnin is None:
@@ -196,53 +298,213 @@ def sample_model(model, input_data, output_folder, nmr_samples=None, burnin=None
     if thinning is None:
         thinning = settings['thinning']
 
-    if not isinstance(initialization_data, InitializationData) and initialization_data is not None:
-        initialization_data = SimpleInitializationData(**initialization_data)
-
-    if not mdt.utils.check_user_components():
-        init_user_settings(pass_if_exists=True)
-
     if isinstance(model, str):
-        model = get_model(model)()
+        model_instance = get_model(model)()
+    else:
+        model_instance = model
+
+    initialization_data = SimpleInitializationData(**initialization_data)
+    initialization_data.apply_to_model(model_instance, input_data)
 
     if post_processing:
-        model.update_active_post_processing('sampling', post_processing)
+        model_instance.update_active_post_processing('sampling', post_processing)
 
-    if isinstance(model, DMRICascadeModelInterface):
-        raise ValueError('The function \'sample_model()\' does not accept cascade models.')
-
-    if cl_device_ind is not None and not isinstance(cl_device_ind, collections.Iterable):
-        cl_device_ind = [cl_device_ind]
-
-    if cl_device_ind is None:
-        cl_context_action = mot.configuration.VoidConfigurationAction()
-    else:
-        cl_envs = [get_cl_devices()[ind] for ind in cl_device_ind]
-        cl_context_action = mot.configuration.RuntimeConfigurationAction(
-            cl_environments=cl_envs,
-            load_balancer=EvenDistribution(),
-            double_precision=double_precision)
-
-    with mot.configuration.config_context(cl_context_action):
-        base_dir = os.path.join(output_folder, model.name, 'samples')
-
-        if not os.path.isdir(base_dir):
-            os.makedirs(base_dir)
-
-        if recalculate:
-            shutil.rmtree(base_dir)
-
-        logger = logging.getLogger(__name__)
-        logger.info('Using MDT version {}'.format(__version__))
-        logger.info('Preparing for model {0}'.format(model.name))
-        logger.info('The parameters we will sample are: {0}'.format(model.get_free_param_names()))
-
-        return sample_composite_model(model, input_data, base_dir, nmr_samples, thinning, burnin,
+    with mot.configuration.config_context(CLRuntimeAction(cl_runtime_info)):
+        from mdt.lib.processing.model_sampling import sample_composite_model
+        return sample_composite_model(model_instance, input_data, output_folder, nmr_samples, thinning, burnin,
                                       get_temporary_results_dir(tmp_results_dir),
                                       method=method, recalculate=recalculate,
                                       store_samples=store_samples,
                                       sample_items_to_save=sample_items_to_save,
-                                      initialization_data=initialization_data)
+                                      post_sampling_cb=post_sampling_cb,
+                                      sampler_options=sampler_options)
+
+
+def compute_fim(model, input_data, optimization_results, output_folder=None, cl_device_ind=None, cl_load_balancer=None,
+                initialization_data=None):
+    """Compute the Fisher Information Matrix (FIM).
+
+    This is typically done as post-processing step during the model fitting process, but can also be performed
+    separately after optimization.
+
+    Since the FIM depends on which parameters were optimized, results will change if different parameters are fixed.
+    That is, this function will compute the FIM for every estimable parameter (free-non-fixed parameters). If you want
+    to have the exact same FIM results as when you computed the FIM as optimization post-processing it is important
+    to have exactly the same maps fixed.
+
+    Contrary to the post-processing of the optimization maps, all FIM results are written to a single sub-folder in the
+    provided output folder.
+
+    Args:
+        model (str or :class:`~mdt.models.base.EstimableModel`):
+            The name of a composite model or an implementation of a composite model.
+        input_data (:class:`~mdt.lib.input_data.MRIInputData`): the input data object containing all
+            the info needed for the model fitting.
+        optimization_results (dict or str): the optimization results, either a dictionary with results or the
+            path to a folder.
+        output_folder (string): Optionally, the path to the folder where to place the output
+        cl_device_ind (List[Union[mot.lib.cl_environments.CLEnvironment, int]]
+                             or mot.lib.cl_environments.CLEnvironment or int): the CL devices to use.
+            Either provide MOT CLEnvironment's or indices from into the list from the function mdt.get_cl_devices().
+        cl_load_balancer (mot.lib.load_balancers.LoadBalancer or Tuple[float]): the load balancer to use. Can also
+            be an array of fractions (summing to 1) with one fraction per device. For example, for two devices one
+            can specify ``cl_load_balancer = [0.3, 0.7]`` to let one device to more work than another.
+        initialization_data (dict): provides (extra) initialization data to
+            use during model fitting. This dictionary can contain the following elements:
+
+            * ``inits``: dictionary with per parameter an initialization point
+            * ``fixes``: dictionary with per parameter a fixed point, this will remove that parameter from the fitting
+            * ``lower_bounds``: dictionary with per parameter a lower bound
+            * ``upper_bounds``: dictionary with per parameter a upper bound
+            * ``unfix``: a list of parameters to unfix
+
+            For example::
+
+                initialization_data = {
+                    'fixes': {'Stick0.theta: np.array(...), ...},
+                    'inits': {...}
+                }
+
+    Returns:
+        dict: all the computed FIM maps in a flattened dictionary.
+    """
+    initialization_data = initialization_data or {}
+
+    if isinstance(optimization_results, str):
+        optimization_results = get_all_nifti_data(optimization_results)
+
+    if not check_user_components():
+        init_user_settings(pass_if_exists=True)
+
+    cl_runtime_info = CLRuntimeInfo(cl_environments=cl_device_ind,
+                                    double_precision=True,
+                                    load_balancer=cl_load_balancer)
+
+    if isinstance(model, str):
+        model_name = model
+        model_instance = get_model(model)()
+    else:
+        model_name = model.name
+        model_instance = model
+
+    model_instance.set_input_data(input_data)
+
+    initialization_data = SimpleInitializationData(**initialization_data)
+    initialization_data.apply_to_model(model_instance, input_data)
+
+    with mot.configuration.config_context(CLRuntimeAction(cl_runtime_info)):
+        opt_points = create_roi(optimization_results, input_data.mask)
+        opt_array = combine_dict_to_array(opt_points, model_instance.get_free_param_names())
+
+        covars = model_instance.compute_covariance_matrix(opt_array)
+        covariance_names = model_instance.get_covariance_output_names()
+
+        return_results = {}
+        for ind, name in enumerate(covariance_names):
+            if name.endswith('.std'):
+                return_results[name] = np.nan_to_num(np.sqrt(covars[..., ind]))
+            else:
+                return_results[name] = covars[..., ind]
+
+        return_results = restore_volumes(return_results, input_data.mask)
+        write_volume_maps(return_results, os.path.join(output_folder, model_name, 'FIM'))
+
+        return return_results
+
+
+def bootstrap_model(model, input_data, optimization_results, output_folder, bootstrap_method=None,
+                    bootstrap_options=None, nmr_samples=None, optimization_method=None, optimizer_options=None,
+                    recalculate=False, cl_device_ind=None, double_precision=False, keep_samples=True,
+                    tmp_results_dir=True, initialization_data=None):
+    """Resample the model using residual bootstrapping.
+
+    This is typically used to construct confidence intervals on the optimized parameters.
+
+    Args:
+        model (str or :class:`~mdt.models.base.EstimableModel`): the model to sample
+        input_data (:class:`~mdt.lib.input_data.MRIInputData`): the input data object containing all
+            the info needed for the model fitting.
+        optimization_results (dict or str): the optimization results, either a dictionary with results or the
+            path to a folder.
+        output_folder (string): The path to the folder where to place the output, we will make a subdir with the
+            model name in it (for the optimization results) and then a subdir with the samples output.
+        bootstrap_method (str): the bootstrap method we want to use, 'residual', or 'wild'. Defaults to 'wild'.
+        bootstrap_options (dict): bootstrapping options specific for the bootstrap method in use
+        nmr_samples (int): the number of samples we would like to compute. Defaults to 1000.
+        optimization_method (str): The optimization method to use, one of:
+            - 'Levenberg-Marquardt'
+            - 'Nelder-Mead'
+            - 'Powell'
+            - 'Subplex'
+
+            If not given, defaults to 'Powell'.
+        optimizer_options (dict): extra options passed to the optimization routines.
+        recalculate (boolean): If we want to recalculate the results if they are already present.
+        cl_device_ind (int): the index of the CL device to use. The index is from the list from the function
+            utils.get_cl_devices().
+        double_precision (boolean): if we would like to do the calculations in double precision
+        keep_samples (boolean): determines if we keep any of the chains. If set to False, the chains will
+            be discarded after generating the mean and standard deviations.
+        tmp_results_dir (str, True or None): The temporary dir for the calculations. Set to a string to use
+                that path directly, set to True to use the config value, set to None to disable.
+        initialization_data (dict): provides (extra) initialization data to
+            use during model fitting. This dictionary can contain the following elements:
+
+            * ``inits``: dictionary with per parameter an initialization point
+            * ``fixes``: dictionary with per parameter a fixed point, this will remove that parameter from the fitting
+            * ``lower_bounds``: dictionary with per parameter a lower bound
+            * ``upper_bounds``: dictionary with per parameter a upper bound
+            * ``unfix``: a list of parameters to unfix
+
+            For example::
+
+                initialization_data = {
+                    'fixes': {'Stick0.theta: np.array(...), ...},
+                    'inits': {...}
+                }
+
+    Returns:
+        dict: if keep_samples is True we return the samples per parameter as a numpy memmap.
+            If store_samples is False we return None
+    """
+    initialization_data = initialization_data or {}
+    nmr_samples = nmr_samples or 1000
+    bootstrap_method = bootstrap_method or 'wild'
+
+    if not check_user_components():
+        init_user_settings(pass_if_exists=True)
+
+    if cl_device_ind is None:
+        cl_context_action = mot.configuration.VoidConfigurationAction()
+    else:
+        cl_context_action = mot.configuration.RuntimeConfigurationAction(
+            cl_environments=get_cl_devices(cl_device_ind),
+            double_precision=double_precision)
+
+    if isinstance(model, str):
+        model_name = model
+        model_instance = get_model(model)()
+    else:
+        model_name = model.name
+        model_instance = model
+
+    model_instance.update_active_post_processing('optimization', {'uncertainties': False, 'll_and_ic': False})
+
+    initialization_data = SimpleInitializationData(**initialization_data)
+    initialization_data.apply_to_model(model_instance, input_data)
+
+    if optimization_method is None:
+        optimization_method, optimizer_options = get_optimizer_for_model(model_name)
+
+    with mot.configuration.config_context(cl_context_action):
+        from mdt.lib.processing.model_bootstrapping import compute_bootstrap
+        return compute_bootstrap(model_instance, input_data, optimization_results,
+                                 output_folder, bootstrap_method, optimization_method, nmr_samples,
+                                 get_temporary_results_dir(tmp_results_dir),
+                                 recalculate=recalculate,
+                                 keep_samples=keep_samples,
+                                 optimizer_options=optimizer_options,
+                                 bootstrap_options=bootstrap_options)
 
 
 def batch_fit(data_folder, models_to_fit, output_folder=None, batch_profile=None,
@@ -314,7 +576,7 @@ def batch_fit(data_folder, models_to_fit, output_folder=None, batch_profile=None
         cl_device_ind=cl_device_ind, double_precision=double_precision,
         tmp_results_dir=tmp_results_dir, use_gradient_deviations=use_gradient_deviations)
 
-    return batch_apply(batch_fit_func, data_folder, batch_profile=batch_profile, subjects_selection=subjects_selection)
+    return batch_apply(data_folder, batch_fit_func, batch_profile=batch_profile, subjects_selection=subjects_selection)
 
 
 def view_maps(data, config=None, figure_options=None,
@@ -461,7 +723,7 @@ def sort_maps(input_maps, reversed_sort=False, sort_index_matrix=None):
         list: the list of sorted volumes
     """
     if sort_index_matrix is None:
-        sort_index_matrix = create_sort_matrix(input_maps, reversed_sort=reversed_sort)
+        sort_index_matrix = create_4d_sort_matrix(input_maps, reversed_sort=reversed_sort)
     elif isinstance(sort_index_matrix, str):
         sort_index_matrix = np.round(load_nifti(sort_index_matrix).get_data()).astype(np.int64)
     return sort_volumes_per_voxel(input_maps, sort_index_matrix)
@@ -495,15 +757,13 @@ def write_volume_maps(maps, directory, header=None, overwrite_volumes=True, gzip
 
 
 def get_models_list():
-    """Get a list of all available models, composite and cascade.
+    """Get a list of all available composite models
 
     Returns:
         list of str: A list of available model names.
     """
-    from mdt.lib.components import list_composite_models, list_cascade_models
-    l = list(list_cascade_models())
-    l.extend(list_composite_models())
-    return list(sorted(l))
+    from mdt.lib.components import get_component_list
+    return list(sorted(get_component_list('composite_models')))
 
 
 def get_models_meta_info():
@@ -513,13 +773,8 @@ def get_models_meta_info():
         dict of dict: The first dictionary indexes the model names to the meta tags, the second holds the meta
             information.
     """
-    from mdt.lib.components import list_cascade_models, list_composite_models, get_meta_info, get_component_list
-    meta_info = {}
-    for model_type in ('composite_models', 'cascade_models'):
-        model_list = get_component_list(model_type)
-        for model in model_list:
-            meta_info.update({model: get_meta_info(model_type, model)})
-    return meta_info
+    from mdt.lib.components import get_meta_info, get_component_list
+    return {model: get_meta_info('composite_models', model) for model in get_component_list('composite_models')}
 
 
 def start_gui(base_dir=None, app_exec=True):
