@@ -6,7 +6,7 @@ from mdt.cli_scripts.mdt_gui import GUI
 __author__ = 'Robbert Harms'
 __date__ = "2015-08-18"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class GUI_Shortcut(GUI):
