@@ -1,4 +1,3 @@
-import inspect
 import re
 from copy import deepcopy
 import numpy as np
@@ -6,15 +5,18 @@ import tatsu
 
 from mdt.component_templates.base import ComponentBuilder, ComponentTemplate
 from mdt.lib.components import get_component
+from mdt.model_building.parameters import FreeParameter, ProtocolParameter
 from mdt.models.composite import DMRICompositeModel
 from mot.lib.cl_function import CLFunction, SimpleCLFunction
 from mdt.model_building.trees import CompartmentModelTree
 import collections
 
+from mot.optimize.base import SimpleConstraintFunction
+
 __author__ = 'Robbert Harms'
 __date__ = "2017-02-14"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 _composite_model_expression_parser = tatsu.compile('''
@@ -30,7 +32,7 @@ _composite_model_expression_parser = tatsu.compile('''
 
 class DMRICompositeModelBuilder(ComponentBuilder):
 
-    def create_class(self, template):
+    def _create_class(self, template):
         """Creates classes with as base class DMRICompositeModel
 
         Args:
@@ -39,18 +41,14 @@ class DMRICompositeModelBuilder(ComponentBuilder):
         """
         class AutoCreatedDMRICompositeModel(DMRICompositeModel):
 
-            def __init__(self, model_name=None, **kwargs):
-                model_name = model_name or deepcopy(template.name)
-
-                keywords = {'signal_noise_model': deepcopy(template.signal_noise_model),
-                            'enforce_weights_sum_to_one': template.enforce_weights_sum_to_one}
-                keywords.update(kwargs)
-
+            def __init__(self, volume_selection=True):
                 super().__init__(
-                    model_name,
+                    deepcopy(template.name),
                     CompartmentModelTree(parse_composite_model_expression(template.model_expression)),
                     deepcopy(_resolve_likelihood_function(template.likelihood_function)),
-                    **keywords
+                    signal_noise_model=_resolve_signal_noise_model(template.signal_noise_model),
+                    enforce_weights_sum_to_one=template.enforce_weights_sum_to_one,
+                    volume_selection=volume_selection
                 )
 
                 for full_param_name, value in template.inits.items():
@@ -67,27 +65,23 @@ class DMRICompositeModelBuilder(ComponentBuilder):
 
                 self.nmr_parameters_for_bic_calculation = self.get_nmr_parameters()
 
-                if template.sort_maps:
-                    self._post_optimization_modifiers.append(_get_map_sorting_modifier(
-                        template.sort_maps, self._model_functions_info.get_model_parameter_list()))
-
-                self._post_optimization_modifiers.extend(_get_model_post_optimization_modifiers(
-                    self._model_functions_info.get_model_list()))
-                self._post_optimization_modifiers.extend(deepcopy(template.post_optimization_modifiers))
-
                 self._extra_optimization_maps_funcs.extend(_get_model_extra_optimization_maps_funcs(
-                    self._model_functions_info.get_model_list()))
+                    self._model_functions_info.get_compartment_models()))
                 self._extra_optimization_maps_funcs.extend(deepcopy(template.extra_optimization_maps))
 
                 self._extra_sampling_maps_funcs.extend(_get_model_extra_sampling_maps_funcs(
-                    self._model_functions_info.get_model_list()))
+                    self._model_functions_info.get_compartment_models()))
                 self._extra_sampling_maps_funcs.extend(deepcopy(template.extra_sampling_maps))
 
-                self._proposal_callbacks.extend(_get_model_proposal_callbacks(
-                    self._model_functions_info.get_model_list()))
-
                 self._model_priors.extend(_resolve_model_prior(
-                    template.extra_prior, self._model_functions_info.get_model_parameter_list()))
+                    template.prior, self._model_functions_info.get_model_parameter_list()))
+
+                constraint_func = _resolve_constraints(
+                    template.constraints, self._model_functions_info.get_model_parameter_list())
+                if constraint_func:
+                    self._constraints.append(constraint_func)
+                self._constraints.extend(_get_compartment_constraints(
+                    self._model_functions_info.get_compartment_models()))
 
             def _get_suitable_volume_indices(self, input_data):
                 volume_selection = template.volume_selection
@@ -95,27 +89,12 @@ class DMRICompositeModelBuilder(ComponentBuilder):
                 if not volume_selection:
                     return super()._get_suitable_volume_indices(input_data)
 
-                use_unweighted = volume_selection.get('use_unweighted', True)
-                use_weighted = volume_selection.get('use_weighted', True)
-                unweighted_threshold = volume_selection.get('unweighted_threshold', 25e6)
+                protocol_indices = []
+                for protocol_name, ranges in volume_selection.items():
+                    values = input_data.protocol[protocol_name]
 
-                protocol = input_data.protocol
-
-                if protocol.has_column('g') and protocol.has_column('b'):
-                    if use_weighted:
-                        if 'min_bval' in volume_selection and 'max_bval' in volume_selection:
-                            protocol_indices = protocol.get_indices_bval_in_range(start=volume_selection['min_bval'],
-                                                                                  end=volume_selection['max_bval'])
-                        else:
-                            protocol_indices = protocol.get_weighted_indices(unweighted_threshold)
-                    else:
-                        protocol_indices = []
-
-                    if use_unweighted:
-                        protocol_indices = list(protocol_indices) + \
-                                           list(protocol.get_unweighted_indices(unweighted_threshold))
-                else:
-                    return list(range(protocol.length))
+                    for start, end in ranges:
+                        protocol_indices.extend(np.where((start <= values) * (values <= end))[0])
 
                 return np.unique(protocol_indices)
 
@@ -126,7 +105,7 @@ class DMRICompositeModelBuilder(ComponentBuilder):
 
 
 class CompositeModelTemplate(ComponentTemplate):
-    """The cascade config to inherit from.
+    """The composite model config to inherit from.
 
     These configs are loaded on the fly by the DMRICompositeModelBuilder
 
@@ -135,55 +114,17 @@ class CompositeModelTemplate(ComponentTemplate):
 
         description (str): model description
 
-        post_optimization_modifiers (list): a list of modification callbacks to change the estimated
-            optimization points. This should not add new maps, for that use the ``additional_results`` directive.
-            This can be used to, for example, sort maps into a similar valued but sorted representation
-            (actually the ``sort_maps`` directive creates a post_optimization_modifier)
-
-            Examples:
-
-            .. code-block:: python
-
-                post_optimization_modifiers = [lambda d: reorient_tensor(d),
-                                               lambda d: {'w_stick0.w': ..., 'w_stick1.w': ...}
-                                                ...]
-
-            This should return a dictionary with updated maps.
-
-        sort_maps (list of tuple): The maps to sort voxel-by voxel as post-processing.
-            To ensure that the point estimate from optimization can directly be used in MCMC sample, we sometimes
-            need to rearrange the weights (and therefore the corresponding compartments). For example, some composite
-            models have a prior on the weights of similar compartments to restrict them to a decreasing order, which
-            is typically done to prevent bimodal continuous_distributions. This model directive allows you to easily specify
-            which parameters to rearrange as first post-processing of optimization results.
-
-            The first tuple needs to be a parameter reference or a  ``Weight`` compartment, the other tuples can
-            contain either parameters or compartments.
-
-            Example input::
-
-                sort_maps = [('w0.w', 'w1.w'), ('Stick0', 'Stick1')]
-
-            this input sorts the weights w0.w and w1.w and afterwards sorts all the parameters of the Stick0 and Stick1
-            compartment according to the sorting of those weights.
-
-            One can also sort just one parameter of a compartment using::
-
-                sort_maps = [('w0', 'w1'), ('Stick0.theta', 'Stick1.theta')]
-
-            which will again sort the weights but will only apply the sorting on the theta map of both the Stick
-            compartments.
-
         extra_optimization_maps (list): a list of functions to return extra information maps based on a point estimate.
-            This is called after the post optimization modifiers and after the model calculated uncertainties based
-            on the Fisher Information Matrix. Therefore, these routines can propagate uncertainties in the estimates.
-            Please note that this is only for adding additional maps. For changing the point estimate of the
-            optimization, please use the ``post_optimization_modifiers`` directive.
+            This is called after after the model calculated uncertainties based on the Fisher Information Matrix.
+            Therefore, these routines can propagate uncertainties in the estimates.
+
+            These functions should accept as single argument an object of type
+            :class:`mdt.models.composite.ExtraOptimizationMapsInfo`.
 
             Examples::
 
                 extra_optimization_maps = [lambda d: {'FS': 1 - d['w_ball.w']},
-                                           lambda d, protocol: {'Kurtosis.MK': <...>},
+                                           lambda d: {'Kurtosis.MK': <...>},
                                            lambda d: {'Power2': d['foo']**2, 'Power3': d['foo']**3},
                                            ...]
 
@@ -238,29 +179,32 @@ class CompositeModelTemplate(ComponentTemplate):
             By default it is True and we add them.
 
         volume_selection (dict): the volume selection by this model. This can be used to limit the volumes used
-            in the analysis to only the volumes included in the specification.
-            Set to None, or an empty dict to disable.
-            The options available are:
+            in the analysis to only the volumes included in the specification. You can specify specific protocol
+            names here for limiting the selected volumes. For example, for the Tensor model we can write::
 
-               * ``unweighted_threshold`` (float): the threshold differentiating between
-                 weighted and unweighted volumes
-               * ``use_unweighted`` (bool): if we want to use unweighted volumes or not
-               * ``use_weighted`` (bool): if we want to use the diffusion weigthed volumes or not
-               * ``min_bval`` (float): the minimum b-value to include
-               * ``max_bval`` (float): the maximum b-value to include
+                volume_selection = {'b': [(0, 1.5e9 + 0.1e9)]}
 
+            To limit the volumes to the b-values between 0 and 1.6e9.
             If the method ``_get_suitable_volume_indices`` is overwritten, this does nothing.
 
-        prior (str, mdt.model_building.utils.ModelPrior or None): a model wide prior. This is used in conjunction with
-            the compartment priors and the parameter priors. If a string is given we will automatically construct a
-            :class:`mdt.model_building.utils.ModelPrior` from that string.
+        prior (str or CLFunction or None): a model wide prior. This is used in conjunction
+            with the compartment priors and the parameter priors.
+
+        constraints (str or None): additional inequality constraints for this model. Each constraint needs to be
+            implemented as ``g(x)`` where we assume that ``g(x) <= 0``. For example, to implement a simple inequality
+            constraint like ``Tensor.d >= Tensor.dperp0``, we first write it as ``Tensor.dperp0 - Tensor.d <= 0``.
+            We can then implement it as::
+
+                constraints = 'constraints[0] = Tensor.dperp0 - Tensor.d;'
+
+            To add more constraint, add another entry to the ``constraints`` array. MDT parses the given text and
+            automatically recognizes the model parameter names and the number of constraints.
     """
     _component_type = 'composite_models'
     _builder = DMRICompositeModelBuilder()
 
     name = ''
     description = ''
-    post_optimization_modifiers = []
     extra_optimization_maps = []
     extra_sampling_maps = []
     model_expression = ''
@@ -272,8 +216,8 @@ class CompositeModelTemplate(ComponentTemplate):
     lower_bounds = {}
     enforce_weights_sum_to_one = True
     volume_selection = None
-    extra_prior = None
-    sort_maps = None
+    prior = None
+    constraints = None
 
     @classmethod
     def meta_info(cls):
@@ -284,7 +228,7 @@ class CompositeModelTemplate(ComponentTemplate):
 
 
 def _resolve_likelihood_function(likelihood_function):
-    """Resolve the likelihood function from string if necessary.
+    """Resolve the likelihood function from a string if necessary.
 
     The composite models accept likelihood functions as a string and as a object. This function
     resolves the strings if a string is given, else it returns the object passed.
@@ -296,11 +240,29 @@ def _resolve_likelihood_function(likelihood_function):
         mdt.model_building.likelihood_models.LikelihoodFunction: the likelihood function to use
     """
     if isinstance(likelihood_function, str):
-        return get_component('likelihood_functions', likelihood_function + 'LikelihoodFunction')()
+        return get_component('likelihood_functions', likelihood_function)()
     else:
         return likelihood_function
 
 
+def _resolve_signal_noise_model(signal_noise_function):
+    """Resolve the signal noise function from a string if necessary.
+
+    The composite models accept signal noise functions as a string and as a object. This function
+    resolves the strings if a string is given, else it returns the object passed.
+
+    Args:
+        signal_noise_function (str or object): the signal noise function to resolve to an object
+
+    Returns:
+        mdt.model_building.signal_noise_models.SignalNoiseModel: the signal noise function to use
+    """
+    if isinstance(signal_noise_function, str):
+        return get_component('signal_noise_functions', signal_noise_function)()
+    else:
+        return signal_noise_function
+
+
 def _resolve_model_prior(prior, model_parameters):
     """Resolve the model priors.
 
@@ -317,113 +279,113 @@ def _resolve_model_prior(prior, model_parameters):
     if isinstance(prior, CLFunction):
         return [prior]
 
+    dotted_names = ['{}.{}'.format(m.name, p.name) for m, p in model_parameters]
+    dotted_names.sort(key=len, reverse=True)
+
     parameters = []
-    for m, p in model_parameters:
-        dotted_name = '{}.{}'.format(m.name, p.name)
+    remaining_prior = prior
+    for dotted_name in dotted_names:
         bar_name = dotted_name.replace('.', '_')
 
-        if dotted_name in prior:
+        if dotted_name in remaining_prior:
             prior = prior.replace(dotted_name, bar_name)
-            parameters.append(('mot_float_type', dotted_name))
-        elif bar_name in prior:
-            parameters.append(('mot_float_type', dotted_name))
+            remaining_prior = remaining_prior.replace(dotted_name, '')
+            parameters.append('mot_float_type ' + dotted_name)
+        elif bar_name in remaining_prior:
+            remaining_prior = remaining_prior.replace(bar_name, '')
+            parameters.append('mot_float_type ' + dotted_name)
 
     return [SimpleCLFunction('mot_float_type', 'model_prior', parameters, prior)]
 
 
-def _get_map_sorting_modifier(sort_maps, model_list):
-    """Construct the map sorting modification routine for the given maps_to_sort attribute.
+def _get_compartment_constraints(compartments):
+    """Get a list of all the constraint functions defined in the compartments.
+
+    This function will add a wrapper around the constraint functions to make the inputs relative to the
+    compartment model. That it, the constraint functions of the compartments expect the parameter names without the
+    model name, whereas the expected input of the composite constraint functions is with the full model.map name.
 
     Args:
-        sort_maps (list of tuple): the list with compartments/parameters to sort. Sorting is done on the first element and
-            the other maps are sorted based on that indexing.
-        model_list (list of tuple): the list of model, parameter tuples to use for matching names.
-    Returns:
-        tuple: name, modification routine. The modification routine will sort the maps as specified.
-    """
-    def is_param_name(name):
-        return '.' in name
-
-    def get_model_by_name(name):
-        for model, parameter in model_list:
-            if model.name == name:
-                return model
-
-    sort_keys = []
-    for name in sort_maps[0]:
-        for model, parameter in model_list:
-            full_name = '{}.{}'.format(model.name, parameter.name)
-            if model.name == name or full_name == name:
-                sort_keys.append(full_name)
-                break
-
-    other_sort_pairs = []
-    for other_pair in sort_maps[1:]:
-        if is_param_name(other_pair[0]):
-            other_sort_pairs.append(other_pair)
-        else:
-            model = get_model_by_name(other_pair[0])
-            for parameter in model.get_free_parameters():
-                sort_pair = []
-                for model_name in other_pair:
-                    sort_pair.append('{}.{}'.format(model_name, parameter.name))
-                other_sort_pairs.append(sort_pair)
-
-    names = []
-    names.extend(sort_keys)
-    for pairs in other_sort_pairs:
-        names.extend(pairs)
-
-    def map_sorting(results):
-        sort_matrix = np.column_stack([results[map_name] for map_name in sort_keys])
-        ranking = np.atleast_2d(np.squeeze(np.argsort(sort_matrix, axis=1)[:, ::-1]))
-        list_index = np.arange(ranking.shape[0])
-
-        sorted_maps = []
-        sorted_maps.extend(sort_matrix[list_index, ranking[:, ind], None] for ind in range(ranking.shape[1]))
-
-        for pair in other_sort_pairs:
-            sort_matrix = np.column_stack([results[map_name] for map_name in pair])
-            sorted_maps.extend(sort_matrix[list_index, ranking[:, ind], None] for ind in range(ranking.shape[1]))
-
-        return dict(zip(names, sorted_maps))
-
-    return map_sorting
-
-
-def _get_model_post_optimization_modifiers(compartments):
-    """Get a list of all the post optimization modifiers defined in the models.
-
-    This function will add a wrapper around the modification routines to make the input and output maps relative to the
-    model. That it, these functions expect the parameter names without the model name and output map names without
-    the model name, whereas the expected input and output of the modifiers of the model is with the full model.map name.
-
-    Args:
-        compartments (list): the list of compartment models from which to get the modifiers
+        compartments (list): the list of compartment models from which to get the constraints
 
     Returns:
-        list: the list of modification routines taken from the compartment models.
+        List[mot.optimize.base.ConstraintFunction]: list of constraint functions from the compartments.
     """
-    modifiers = []
+    constraints = []
 
-    def get_wrapped_modifier(compartment_name, original_modifier):
-        def get_compartment_specific_maps(results):
-            return {k[len(compartment_name) + 1:]: v for k, v in results.items() if k.startswith(compartment_name)}
+    def get_wrapped_function(compartment_name, original_constraint_func):
+        parameters = []
+        for param in original_constraint_func.get_parameters():
+            if isinstance(param, FreeParameter):
+                parameters.append(param.get_renamed('{}_{}'.format(compartment_name, param.name)))
+            elif isinstance(param, ProtocolParameter):
+                parameters.append(param)
 
-        def prepend_compartment_name(results):
-            return {'{}.{}'.format(compartment_name, key): value for key, value in results.items()}
+        body = original_constraint_func.get_cl_function_name() + \
+               '(' + ', '.join(p.name for p in parameters) + ', constraints);'
 
-        def wrapped_modifier(results):
-            return prepend_compartment_name(original_modifier(get_compartment_specific_maps(results)))
-
-        return wrapped_modifier
+        return SimpleConstraintFunction(
+            'void', 'wrapped_' + original_constraint_func.get_cl_function_name(),
+            parameters + ['local mot_float_type* constraints'],
+            body,
+            dependencies=[original_constraint_func],
+            nmr_constraints=original_constraint_func.get_nmr_constraints())
 
     for compartment in compartments:
-        if hasattr(compartment, 'post_optimization_modifiers'):
-            for modifier in compartment.post_optimization_modifiers:
-                modifiers.append(get_wrapped_modifier(compartment.name, modifier))
+        if compartment.get_constraints_func():
+            constraints.append(get_wrapped_function(compartment.name, compartment.get_constraints_func()))
 
-    return modifiers
+    return constraints
+
+
+def _resolve_constraints(constraint, model_parameters):
+    """Resolve the constraints.
+
+    This parses the given constraints to recognize the parameters and the number of constraints.
+
+    Args:
+        constraint (str): the string with the constraints
+        model_parameters (tuple(str)): the (model, parameter) tuples for all parameters in the model
+
+    Returns:
+        mot.optimize.base.ConstraintFunction: the additional constraint function for this composite model
+    """
+    if constraint is None:
+        return None
+
+    constraint_refs = re.findall(r'constraints\[(\d+)\]', constraint)
+
+    if not constraint_refs:
+        return None
+
+    nmr_constraints = max(map(int, constraint_refs)) + 1
+
+    parameters = []
+    protocol_parameters_seen = []
+    for m, p in model_parameters:
+        if isinstance(p, FreeParameter):
+            parameters.append(p.get_renamed('{}_{}'.format(m.name, p.name)))
+        elif isinstance(p, ProtocolParameter):
+            if p.name not in protocol_parameters_seen:
+                parameters.append(p)
+                protocol_parameters_seen.append(p.name)
+
+    dotted_names = ['{}.{}'.format(m.name, p.name) for m, p in model_parameters]
+    dotted_names.sort(key=len, reverse=True)
+
+    remaining_constraint = constraint
+    for dotted_name in dotted_names:
+        bar_name = dotted_name.replace('.', '_')
+
+        if dotted_name in remaining_constraint:
+            constraint = constraint.replace(dotted_name, bar_name)
+            remaining_constraint = remaining_constraint.replace(dotted_name, '')
+        elif bar_name in remaining_constraint:
+            remaining_constraint = remaining_constraint.replace(bar_name, '')
+
+    return SimpleConstraintFunction('void', 'model_constraints',
+                                    parameters + ['local mot_float_type* constraints'],
+                                    constraint, nmr_constraints=nmr_constraints)
 
 
 def _get_model_extra_optimization_maps_funcs(compartments):
@@ -431,11 +393,11 @@ def _get_model_extra_optimization_maps_funcs(compartments):
 
     This function will add a wrapper around the modification routines to make the input and output maps relative to the
     model. That it, the functions in the compartments expect the parameter names without the model name and they output
-    maps without the model name, whereas the expected input and output of the modifiers of the model is with the
+    maps without the model name, whereas the expected input and output of the functions of the model is with the
     full model.map name.
 
     Args:
-        compartments (list): the list of compartment models from which to get the modifiers
+        compartments (list): the list of compartment models from which to get the extra optimization maps
 
     Returns:
         list: the list of modification routines taken from the compartment models.
@@ -443,31 +405,27 @@ def _get_model_extra_optimization_maps_funcs(compartments):
     funcs = []
 
     def get_wrapped_func(compartment_name, original_func):
-        def get_compartment_specific_maps(results):
+        def get_compartment_specific_results(results):
             maps = {k[len(compartment_name) + 1:]: v for k, v in results.items() if k.startswith(compartment_name)}
 
-            if results['covariances']:
+            if 'covariances' in results and results['covariances'] is not None:
                 p = re.compile(compartment_name + r'\.\w+_to_' + compartment_name + r'\.\w+')
                 maps['covariances'] = {k.replace(compartment_name + '.', ''): v
                                        for k, v in results['covariances'].items() if p.match(k)}
-            return maps
+
+            return results.copy_with_different_results(maps)
 
         def prepend_compartment_name(results):
             return {'{}.{}'.format(compartment_name, key): value for key, value in results.items()}
 
-        if 'input_data' in inspect.signature(original_func).parameters:
-            def wrapped_modifier(results, input_data):
-                return prepend_compartment_name(original_func(get_compartment_specific_maps(results), input_data))
-        else:
-            def wrapped_modifier(results):
-                return prepend_compartment_name(original_func(get_compartment_specific_maps(results)))
+        def wrapped_modifier(results):
+            return prepend_compartment_name(original_func(get_compartment_specific_results(results)))
 
         return wrapped_modifier
 
     for compartment in compartments:
-        if hasattr(compartment, 'extra_optimization_maps_funcs'):
-            for func in compartment.extra_optimization_maps_funcs:
-                funcs.append(get_wrapped_func(compartment.name, func))
+        for func in compartment.get_extra_optimization_maps_funcs():
+            funcs.append(get_wrapped_func(compartment.name, func))
 
     return funcs
 
@@ -477,11 +435,11 @@ def _get_model_extra_sampling_maps_funcs(compartments):
 
     This function will add a wrapper around the modification routines to make the input and output maps relative to the
     model. That it, the functions in the compartments expect the parameter names without the model name and they output
-    maps without the model name, whereas the expected input and output of the modifiers of the model is with the
+    maps without the model name, whereas the expected input and output of the functions of the model is with the
     full model.map name.
 
     Args:
-        compartments (list): the list of compartment models from which to get the modifiers
+        compartments (list): the list of compartment models from which to get the extra sampling maps
 
     Returns:
         list: the list of extra sample routines taken from the compartment models.
@@ -498,34 +456,12 @@ def _get_model_extra_sampling_maps_funcs(compartments):
         return wrapped_modifier
 
     for compartment in compartments:
-        if hasattr(compartment, 'extra_sampling_maps_funcs'):
-            for func in compartment.extra_sampling_maps_funcs:
-                funcs.append(get_wrapped_func(compartment.name, func))
+        for func in compartment.get_extra_sampling_maps_funcs():
+            funcs.append(get_wrapped_func(compartment.name, func))
 
     return funcs
 
 
-def _get_model_proposal_callbacks(compartments):
-    """Get a list of all the additional proposal callback functions defined in the compartments.
-
-    This function will add a wrapper around the proposal callback to make the input and output maps relative to the
-    model.
-
-    Args:
-        compartments (list): the list of compartment models from which to get the modifiers
-
-    Returns:
-        list: the list of proposal callbacks taken from the compartment models.
-    """
-    funcs = []
-    for compartment in compartments:
-        if hasattr(compartment, 'proposal_callbacks'):
-            for params, func in compartment.proposal_callbacks:
-                compartment_params = [(compartment, p) for p in params]
-                funcs.append((compartment_params, func))
-    return funcs
-
-
 class CompartmentContextResults(collections.Mapping):
 
     def __init__(self, compartment_name, input_results):
@@ -555,19 +491,19 @@ def parse_composite_model_expression(model_expression):
     """Parse the given model expression into a suitable model tree.
 
     Args:
-        model_expression (str): the model expression string. Example:
+        model_expression (str): the model expression string. Example::
 
-        .. code-block:: none
-
-            S0 * ( (Weight(Wball) * Ball) +
-                   (Weight(Wstick) * Stick ) )
+            model_expression = '''
+                S0 * ( (Weight(Wball) * Ball) +
+                       (Weight(Wstick) * Stick ) )
+            '''
 
         If the model name is followed by parenthesis the string in parenthesis will represent the model's nickname.
 
     Returns:
         :class:`list`: the compartment model tree for use in composite models.
     """
-    class Semantics(object):
+    class Semantics:
 
         def expr(self, ast):
             if not isinstance(ast, list):
