@@ -22,20 +22,18 @@ import os
 import re
 from copy import deepcopy
 
-import collections
 import yaml
 from contextlib import contextmanager
 from pkg_resources import resource_stream
 
 import mot.configuration
-from mot.lib.load_balance_strategies import EvenDistribution
 
 from mdt.__version__ import __version__
 
 __author__ = 'Robbert Harms'
 __date__ = "2015-06-23"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 """ The current configuration """
 _config = {}
@@ -70,7 +68,7 @@ def set_config_option(option_name, value):
 
     Args:
         option_name (list of str or str): the name of the option, or a path to the option.
-        value (object): the object to set for that option
+        value : the object to set for that option
 
     Returns:
         object: the raw configuration value defined for that option
@@ -100,7 +98,7 @@ def _config_insert(keys, value):
 
     Args:
         keys (list of str): the position of the input value
-        value (object): the value to put at the position of the key.
+        value : the value to put at the position of the key.
     """
     config = _config
     for key in keys[:-1]:
@@ -230,7 +228,7 @@ def update_write_config(config_file, update_dict):
         yaml.safe_dump(config_dict, f)
 
 
-class ConfigSectionLoader(object):
+class ConfigSectionLoader:
 
     def load(self, value):
         """Load the given configuration value into the current configuration.
@@ -329,41 +327,29 @@ class ActivePostProcessingLoader(ConfigSectionLoader):
         sampling['multivariate_ess'] = sampling.get('multivariate_ess', False)
         sampling['maximum_likelihood'] = sampling.get('maximum_likelihood', False)
         sampling['maximum_a_posteriori'] = sampling.get('maximum_a_posteriori', False)
-        sampling['model_defined_maps'] = sampling.get('model_defined_maps', False)
+        sampling['model_defined_maps'] = sampling.get('model_defined_maps', True)
+        sampling['univariate_normal'] = sampling.get('univariate_normal', True)
+        sampling['average_acceptance_rate'] = sampling.get('average_acceptance_rate', False)
 
         optimization = value.get('optimization', {})
         optimization['uncertainties'] = optimization.get('uncertainties', True)
         optimization['store_covariances'] = optimization.get('store_covariances', True)
+        optimization['ll_and_ic'] = optimization.get('ll_and_ic', True)
 
         _config_insert(['active_post_processing', 'optimization'], optimization)
         _config_insert(['active_post_processing', 'sampling'], sampling)
 
 
-class AutomaticCascadeModels(ConfigSectionLoader):
-    """Load the automatic cascade model settings."""
-
-    def load(self, value):
-        _config_insert(['auto_generate_cascade_models', 'enabled'], value.get('enabled', True))
-        _config_insert(['auto_generate_cascade_models', 'excluded'], value.get('excluded', []))
-
-
 class RuntimeSettingsLoader(ConfigSectionLoader):
 
     def load(self, value):
         if 'cl_device_ind' in value:
             if value['cl_device_ind'] is not None:
                 from mdt.utils import get_cl_devices
-                all_devices = get_cl_devices()
-
-                indices = value['cl_device_ind']
-                if not isinstance(indices, collections.Iterable):
-                    indices = [indices]
-
-                devices = [all_devices[ind] for ind in indices if ind < len(all_devices)]
+                devices = get_cl_devices(value['cl_device_ind'])
 
                 if devices:
                     mot.configuration.set_cl_environments(devices)
-                    mot.configuration.set_load_balancer(EvenDistribution())
 
     def update(self, config_dict, updates):
         if 'runtime_settings' not in config_dict:
@@ -401,9 +387,6 @@ def get_section_loader(section):
     if section == 'runtime_settings':
         return RuntimeSettingsLoader()
 
-    if section == 'auto_generate_cascade_models':
-        return AutomaticCascadeModels()
-
     if section == 'active_post_processing':
         return ActivePostProcessingLoader()
 
@@ -456,13 +439,13 @@ def get_processing_strategy(processing_type, *args, **kwargs):
     Args:
         processing_type (str): 'optimization', 'sampling' or any other of the
             processing_strategies defined in the config
-        model_names (list of str): the list of model names (the full recursive cascade of model names)
+        *args: passed to the constructor of the loaded processing strategy.
         **kwargs: passed to the constructor of the loaded processing strategy.
 
     Returns:
         ModelProcessingStrategy: the processing strategy to use for this model
     """
-    from mdt.lib.processing_strategies import VoxelRange
+    from mdt.lib.processing.processing_strategies import VoxelRange
     options = _config['processing_strategies'].get(processing_type, {}) or {}
     options.update(kwargs)
     return VoxelRange(*args, **options)
@@ -480,34 +463,21 @@ def get_logging_configuration_dict():
     return _config['logging']['info_dict']
 
 
-def get_general_optimizer():
-    """Load the general optimizer from the configuration.
-
-    Returns:
-        Optimizer: the configured optimizer for use in MDT
-    """
-    return _config['optimization']['general']['name']
-
-
-def get_optimizer_for_model(model_names):
-    """Get the optimizer for this specific cascade of models.
-
-    This configuration function supports having a different optimizer for optimizing, for example, NODDI in a
-    cascade and NODDI without a cascade.
+def get_optimizer_for_model(model_name):
+    """Get the optimizer for this specific model.
 
     Args:
-        model_names (list of str): the list of model names (typically a cascade of models) for which we
-            want to get the optimizer to use.
+        model_name (str): the name of the composite model for which we want to get the optimizer to use.
 
     Returns:
         Optimizer: the optimizer to use for optimizing the specific model
     """
-    info_dict = get_model_config(model_names, _config['optimization']['model_specific'])
+    info_dict = get_model_config(model_name, _config['optimization']['model_specific'])
 
-    if info_dict:
-        return info_dict['name']
-    else:
-        return get_general_optimizer()
+    if not info_dict:
+        info_dict = _config['optimization']['general']
+
+    return info_dict['name'], info_dict['options']
 
 
 def get_general_optimizer_name():
@@ -537,95 +507,25 @@ def get_general_sampling_settings():
     return _config['sampling']['general']['settings']
 
 
-def use_automatic_generated_cascades():
-    """Check if we want to use the automatic cascade generation in MDT.
-
-    Returns:
-        boolean: True if we want to use the automatic cascades, False otherwise
-    """
-    return _config['auto_generate_cascade_models']['enabled']
-
-
-def get_automatic_generated_cascades_excluded():
-    """Get the information about the model names excluded from automatic cascade generation.
-
-    Returns:
-        list: the names of the composite models we want to exclude from automatic cascade generation
-    """
-    return _config['auto_generate_cascade_models']['excluded']
-
-
-def get_model_config(model_names, config):
+def get_model_config(model_name, config):
     """Get from the given dictionary the config for the given model.
 
-    This tries to find the best match between the given config items (by key) and the given model list. For example
-    if model_names is ['BallStick', 'S0'] and we have the following config dict:
-
-    .. code-block:: python
-
-        {'^S0$': 0,
-         '^BallStick$': 1
-         ('^BallStick$', '^S0$'): 2,
-         ('^BallStickStick$', '^BallStick$', '^S0$'): 3,
-         }
-
-    then this function should return 2. because that one is the best match, even though the last option is also a
-    viable match. That is, since a subset of the keys in the last option also matches the model names, it is
-    considered a match as well. Still the best match is the third option (returning 2).
+    This tries to find the best match between the given config items (by key) and the given model name.
 
     Args:
-        model_names (list of str): the names of the models we want to match. This should contain the entire
-            recursive list of cascades leading to the composite model we want to get the config for.
-        config (dict): the config items with as keys either a composite model regex for a name or a list of regex for
-            a chain of model names.
+        model_name (str): the name of the model we want to match.
+        config (dict): the config items with as keys a composite model regex
 
     Returns:
-        The config content of the best matching key.
+        The config content of a matching key.
     """
     if not config:
         return {}
 
-    def get_key_length(key):
-        if isinstance(key, tuple):
-            return len(key)
-        return 1
-
-    def is_match(model_names, config_key):
-        if isinstance(model_names, str):
-            model_names = [model_names]
-
-        if len(model_names) != get_key_length(config_key):
-            return False
-
-        if isinstance(config_key, tuple):
-            return all([re.match(config_key[ind], model_names[ind]) for ind in range(len(config_key))])
-
-        return re.match(config_key, model_names[0])
-
-    key_length_lookup = ((get_key_length(key), key) for key in config.keys())
-    ascending_keys = tuple(item[1] for item in sorted(key_length_lookup, key=lambda info: info[0]))
-
-    # direct matching
-    for key in ascending_keys:
-        if is_match(model_names, key):
+    for key in config.keys():
+        if re.match(key, model_name):
             return config[key]
 
-    # partial matching string keys to last model name
-    for key in ascending_keys:
-        if not isinstance(key, tuple):
-            if is_match(model_names[-1], key):
-                return config[key]
-
-    # partial matching tuple keys with a moving filter
-    for key in ascending_keys:
-        if isinstance(key, tuple):
-            for start_ind in range(len(key)):
-                sub_key = key[start_ind:]
-
-                if is_match(model_names, sub_key):
-                    return config[key]
-
-    # no match found
     return {}
 
 
@@ -662,8 +562,8 @@ def config_context(config_action):
     This loads the configuration from a YAML string and uses that configuration as the context.
 
     Args:
-        config_action (ConfigAction or str): the configuration action to apply. If a string is given we will
-            use it using the YamlStringAction config action.
+        config_action (mdt.configuration.ConfigAction or str): the configuration action to apply.
+            If a string is given we will use it using the YamlStringAction config action.
     """
     if isinstance(config_action, str):
         config_action = YamlStringAction(config_action)
@@ -673,7 +573,7 @@ def config_context(config_action):
     config_action.unapply()
 
 
-class ConfigAction(object):
+class ConfigAction:
 
     def __init__(self):
         """Defines a configuration action for the use in a configuration context.
