@@ -1,9 +1,10 @@
 from mdt import CompartmentTemplate
 
+
 __author__ = 'Robbert Harms'
 __date__ = "2015-06-21"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class Stick(CompartmentTemplate):
