@@ -3,7 +3,7 @@ from mdt import WeightCompartmentTemplate
 __author__ = 'Robbert Harms'
 __date__ = '2018-03-23'
 __maintainer__ = 'Robbert Harms'
-__email__ = 'robbert.harms@maastrichtuniversity.nl'
+__email__ = 'robbert@xkls.nl'
 __licence__ = 'LGPL v3'
 
 
@@ -16,7 +16,7 @@ class Weight(WeightCompartmentTemplate):
 class ARD_Beta_Weight(Weight):
     """A weight compartment with a Beta distribution prior between [0, 1].
 
-    Meant for use in Automatic Relevance Detection.
+    Meant for use in Automatic Relevance Detection MCMC sampling.
     """
     parameters = ('w_ard_beta(w)',)
 
@@ -24,6 +24,6 @@ class ARD_Beta_Weight(Weight):
 class ARD_Gaussian_Weight(Weight):
     """A weight compartment with a Gaussian prior with mean at zero and std given by a hyperparameter.
 
-    Meant for use in Automatic Relevance Detection.
+    Meant for use in Automatic Relevance Detection MCMC sampling.
     """
     parameters = ('w_ard_gaussian(w)',)
