@@ -3,12 +3,11 @@ from mdt import LibraryFunctionTemplate
 __author__ = 'Robbert Harms'
 __date__ = "2015-06-21"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class MRIConstants(LibraryFunctionTemplate):
-
-    description = 'Some constants that might be of use in dMRI model functions'
+    """Some constants that might be of use in dMRI model functions"""
     is_function = False
     cl_code = '''
         /**
