@@ -1,4 +1,4 @@
 __author__ = 'Robbert Harms'
 __date__ = "2016-09-03"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
