@@ -16,7 +16,7 @@ from mdt.visualization.maps.base import Zoom, Point2d, DataInfo, Font, MapPlotCo
 __author__ = 'Robbert Harms'
 __date__ = "2016-09-03"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class TabGeneral(QWidget, Ui_TabGeneral):
