@@ -13,7 +13,7 @@ from mdt.visualization.maps.base import DataInfo, SingleMapConfig, MapPlotConfig
 __author__ = 'Robbert Harms'
 __date__ = "2016-09-03"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class TabMapSpecific(QWidget, Ui_TabMapSpecific):
