@@ -3,7 +3,7 @@ from PyQt5.QtWidgets import QDoubleSpinBox
 __author__ = 'Robbert Harms'
 __date__ = "2017-01-26"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class QDoubleSpinBoxDotSeparator(QDoubleSpinBox):
