@@ -19,16 +19,17 @@ from textwrap import dedent
 from mdt.lib.components import get_batch_profile, get_component_list
 from mdt.lib.masking import create_median_otsu_brain_mask
 from mdt.protocols import load_protocol, auto_load_protocol
-from mdt.utils import AutoDict, load_input_data, natural_key_sort_cb
+from mdt.utils import AutoDict, natural_key_sort_cb
+from mdt.lib.input_data import load_input_data
 from mdt.lib.nifti import load_nifti
 
 __author__ = 'Robbert Harms'
 __date__ = "2015-08-21"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
-class BatchProfile(object):
+class BatchProfile:
 
     def get_subjects(self, data_folder):
         """Get the information about all the subjects in the current folder.
@@ -137,7 +138,7 @@ class SimpleBatchProfile(BatchProfile):
             bval_fname=bval_fname, protocol_columns=protocol_columns)
 
 
-class SubjectInfo(object):
+class SubjectInfo:
 
     @property
     def subject_id(self):
@@ -148,6 +149,15 @@ class SubjectInfo(object):
         """
         raise NotImplementedError()
 
+    @property
+    def data_folder(self):
+        """The data folder in which this subject was found.
+
+        Returns:
+            str: the data folder used by the batch profile when loading this subject.
+        """
+        raise NotImplementedError()
+
     @property
     def subject_base_folder(self):
         """Get the data base folder of this subject.
@@ -166,7 +176,7 @@ class SubjectInfo(object):
             use_gradient_deviations (boolean): if we should enable the use of the gradient deviations
 
         Returns:
-            :class:`~mdt.utils.MRIInputData`: the input data to use during model fitting
+            :class:`~mdt.lib.input_data.MRIInputData`: the input data to use during model fitting
         """
         raise NotImplementedError()
 
@@ -174,20 +184,23 @@ class SubjectInfo(object):
         return dedent('''
             {class_name}
                 subject_id: {subject_id}
+                data_folder: {data_folder}
                 subject_base_folder: {subject_base_folder}
                 get_input_data(): <MRIInputData object>
-        '''.format(class_name=self.__class__, subject_id=self.subject_id, subject_base_folder=self.subject_base_folder))
+        '''.format(class_name=self.__class__, subject_id=self.subject_id,
+                   data_folder=self.data_folder, subject_base_folder=self.subject_base_folder))
 
 
 class SimpleSubjectInfo(SubjectInfo):
 
-    def __init__(self, subject_base_folder, subject_id, dwi_fname, protocol_loader,
+    def __init__(self, data_folder, subject_base_folder, subject_id, dwi_fname, protocol_loader,
                  mask_fname, gradient_deviations=None, noise_std=None):
         """This class contains all the information about found subjects during batch fitting.
 
         It is returned by the method get_subjects() from the class BatchProfile.
 
         Args:
+            data_folder (str): the data folder used by the batch profile
             subject_base_folder (str): the base folder of this subject
             subject_id (str): the subject id
             dwi_fname (str): the filename with path to the dwi image
@@ -197,6 +210,7 @@ class SimpleSubjectInfo(SubjectInfo):
             noise_std (float, ndarray, str): either None for automatic noise detection or a float with the noise STD
                 to use during fitting or an ndarray with one value per voxel.
         """
+        self._data_folder = data_folder
         self._subject_base_folder = subject_base_folder
         self._subject_id = subject_id
         self._dwi_fname = dwi_fname
@@ -209,6 +223,10 @@ class SimpleSubjectInfo(SubjectInfo):
     def subject_id(self):
         return self._subject_id
 
+    @property
+    def data_folder(self):
+        return self._data_folder
+
     @property
     def subject_base_folder(self):
         return self._subject_base_folder
@@ -240,7 +258,7 @@ class SimpleSubjectInfo(SubjectInfo):
         return None
 
 
-class BatchSubjectSelection(object):
+class BatchSubjectSelection:
 
     def get_selection(self, subject_ids):
         """Get the selection of subjects from the given list of subjects.
@@ -354,7 +372,7 @@ def get_subject_selection(subjects_selection):
         raise ValueError('Subjects selection should contain either all strings or all integers.')
 
 
-class BatchFitProtocolLoader(object):
+class BatchFitProtocolLoader:
 
     def __init__(self, base_dir, protocol_fname=None, protocol_columns=None, bvec_fname=None, bval_fname=None):
         """A simple protocol loader for loading a protocol from a protocol file or bvec/bval files.
@@ -440,7 +458,7 @@ def get_subject_information(data_folder, subject_ids, batch_profile=None):
     def get_subjects(subject_info):
         subjects.append(subject_info)
 
-    batch_apply(get_subjects, data_folder, batch_profile=batch_profile,
+    batch_apply(data_folder, get_subjects, batch_profile=batch_profile,
                 subjects_selection=SelectedSubjects(subject_ids))
 
     if isinstance(subject_ids, str):
@@ -448,7 +466,7 @@ def get_subject_information(data_folder, subject_ids, batch_profile=None):
     return subjects
 
 
-def batch_apply(func, data_folder, batch_profile=None, subjects_selection=None, extra_args=None):
+def batch_apply(data_folder, func, batch_profile=None, subjects_selection=None, extra_args=None):
     """Apply a function on the subjects found in the batch profile.
 
     Args:
@@ -487,7 +505,7 @@ def batch_apply(func, data_folder, batch_profile=None, subjects_selection=None,
     return results
 
 
-class BatchFitSubjectOutputInfo(object):
+class BatchFitSubjectOutputInfo:
 
     def __init__(self, output_path, subject_id, model_name):
         """This class is used in conjunction with the function :func:`run_function_on_batch_fit_output`.
@@ -509,7 +527,7 @@ class BatchFitSubjectOutputInfo(object):
         '''.format(subject_id=self.subject_id, output_path=self.output_path, model_name=self.model_name)).strip()
 
 
-def run_function_on_batch_fit_output(func, output_folder, subjects_selection=None):
+def run_function_on_batch_fit_output(func, output_folder, subjects_selection=None, model_names=None):
     """Run a function on the output of a batch fitting routine.
 
     This enables you to run a function on every model output from every subject. This expects the output directory
@@ -521,6 +539,8 @@ def run_function_on_batch_fit_output(func, output_folder, subjects_selection=Non
         output_folder (str): The data input folder
         subjects_selection (BatchSubjectSelection or iterable): the subjects to use for processing.
             If None all subjects are processed.
+        model_names (list): the list of model names to process. If not given we will run the function on all
+            models.
 
     Returns:
         dict: indexed by subject->model_name, values are the return values of the users function
@@ -534,8 +554,13 @@ def run_function_on_batch_fit_output(func, output_folder, subjects_selection=Non
 
     results = AutoDict()
     for subject_id in subject_ids:
-        for model_name in os.listdir(os.path.join(output_folder, subject_id)):
+        subject_models = model_names
+        if subject_models is None:
+            subject_models = os.listdir(os.path.join(output_folder, subject_id))
+
+        for model_name in subject_models:
             info = BatchFitSubjectOutputInfo(os.path.join(output_folder, subject_id, model_name),
                                              subject_id, model_name)
             results[subject_id][model_name] = func(info)
+
     return results.to_normal_dict()
