@@ -1,7 +1,7 @@
 __author__ = 'Robbert Harms'
 __date__ = "2016-06-25"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class ProtocolIOError(Exception):
