@@ -1,88 +1,22 @@
 """This module contains various standard post-processing routines for use after optimization or sample."""
 import numpy as np
-from mdt.utils import tensor_spherical_to_cartesian, tensor_cartesian_to_spherical
+from mdt.utils import tensor_spherical_to_cartesian, voxelwise_vector_matrix_vector_product, create_covariance_matrix, \
+    compute_noddi_dti
 from mot.lib.utils import split_in_batches, parse_cl_function
 from mot.lib.kernel_data import Array, Zeros, Scalar
 from mdt.lib.components import get_component
 
-
 __author__ = 'Robbert Harms'
 __date__ = '2017-12-10'
 __maintainer__ = 'Robbert Harms'
-__email__ = 'robbert.harms@maastrichtuniversity.nl'
+__email__ = 'robbert@xkls.nl'
 __licence__ = 'LGPL v3'
 
 
-def noddi_dti_maps(results, input_data=None):
-    """Compute NODDI-like statistics from Tensor/Kurtosis parameter fits.
-
-    Several authors noted correspondence between NODDI parameters and DTI parameters [1, 2]. This function computes
-    the neurite density index (NDI) and NODDI's measure of neurite dispersion using Tensor parameters.
-
-    The corresponding theory assumes that the intrinsic diffusivity of the intra-neurite compartment of NODDI
-    is fixed to d = 1.7 x 10^-9 m^2 s^-1. As such, we fix it here to that value as well and compute the corresponding
-    NODDI-DTI results.
-
-    Args:
-        results (dict): the result dictionary, should contain at least:
-
-            - d (ndarray): principal diffusivity
-            - dperp0 (ndarray): primary perpendicular diffusion
-            - dperp1 (ndarray): primary perpendicular diffusion
-
-            And, if present, we also use these:
-
-            - FA (ndarray): if computed already, the Fractional Anisotropy of the given diffusivities
-            - MD (ndarray): if computed already, the Mean Diffusivity of the given diffusivities
-            - MK (ndarray): if computing for Kurtosis, the computed Mean Kurtosis. If not given, we assume unity.
-
-        input_data (mdt.utils.MRIInputData): optionally, the input data used during the model fitting.
-                If set, we apply the heuristic correction for Diffusional Kurtosis to the MD [1].
-
-    Returns:
-        tuple: the NODDI-DTI, NDI and ODI measures.
-
-    References:
-        1. Edwards LJ, Pine KJ, Ellerbrock I, Weiskopf N, Mohammadi S. NODDI-DTI: Estimating neurite orientation and
-            dispersion parameters from a diffusion tensor in healthy white matter.
-            Front Neurosci. 2017;11(DEC):1-15. doi:10.3389/fnins.2017.00720.
-        2. Lampinen B, Szczepankiewicz F, Martensson J, van Westen D, Sundgren PC, Nilsson M. Neurite density
-            imaging versus imaging of microscopic anisotropy in diffusion MRI: A model comparison using spherical
-            tensor encoding. Neuroimage. 2017;147(July 2016):517-531. doi:10.1016/j.neuroimage.2016.11.053.
-    """
-    noddi_d = 1.7e-9
-
-    d = results['d']
-    dperp0 = results['dperp0']
-    dperp1 = results['dperp1']
-
-    FA = results.get('FA', DTIMeasures.fractional_anisotropy(d, dperp0, dperp1))
-    MD = results.get('MD', (d + dperp0 + dperp1) / 3.)
-
-    tau = 1 / 3. * (1 + (4 / np.fabs(noddi_d - MD)) * (MD * FA / np.sqrt(3 - 2 * FA ** 2)))
-    tau = np.nan_to_num(tau)
-
-    if input_data is not None:
-        shells = input_data.protocol.get_b_values_shells()
-
-        b = shells[0]['b_value']
-        if len(shells) > 1:
-            b = shells[1]['b_value'] - shells[0]['b_value']
-
-        sum = (d ** 2 + dperp0 ** 2 + dperp1 ** 2) / 5 + 2 * (d * dperp0 + d * dperp1 + dperp0 * dperp1) / 15
-
-        MD += ((b / 6) * sum) * results.get('MK', 1)
-
-    ndi = 1 - np.sqrt(0.5 * ((3 * MD) / noddi_d - 1))
-    ndi = np.clip(np.nan_to_num(ndi), 0, 1)
-
-    return {'NODDI_DTI_NDI': ndi, 'NODDI_DTI_TAU': tau}
-
-
-class DTIMeasures(object):
+class DTIMeasures:
 
     @staticmethod
-    def extra_optimization_maps(results, input_data=None):
+    def extra_optimization_maps(results):
         """Return some interesting measures like FA, MD, RD and AD.
 
         This function is meant to be used as a post processing routine in Tensor-like compartment models.
@@ -90,7 +24,6 @@ class DTIMeasures(object):
         Args:
             results (dict): Dictionary containing at least theta, phi, psi, d, dperp0 and dperp1
                 We will use this to generate some standard measures from the diffusion Tensor.
-            input_data (mdt.utils.MRIInputData): optionally, the input data used during the model fitting.
 
         Returns:
             dict: as keys typical elements like 'FA and 'MD' as interesting output and as per values the maps.
@@ -141,10 +74,7 @@ class DTIMeasures(object):
             ('MD', (results['d'] + results['dperp0'] + results['dperp1']) / 3.),
             ('FA', DTIMeasures.fractional_anisotropy(results['d'], results['dperp0'], results['dperp1'])),
             ('RD', (results['dperp0'] + results['dperp1']) / 2.0),
-            ('AD', results['d']),
-            ('d', results['d']),
-            ('dperp0', results['dperp0']),
-            ('dperp1', results['dperp1']),
+            ('AD', results['d'])
         ]
 
         results = {}
@@ -153,28 +83,6 @@ class DTIMeasures(object):
                             name + '.std': np.std(data, axis=1)})
         return results
 
-    @staticmethod
-    def post_optimization_modifier(parameters_dict):
-        """Apply post optimization modification of the Tensor compartment.
-
-        This will re-orient the Tensor such that the eigenvalues are in decreasing order. This is done by
-        permuting the eigen-values and -vectors and then recreating theta, phi and psi to match the rotated system.
-
-        This is done primarily to be able to directly use the Tensor results in MCMC sample. Since we often put a
-        prior on the diffusivities to be in decreasing order, we need to make sure that the starting point is valid.
-
-        Args:
-            parameters_dict (dict): the results from optimization. This expects each value to be a (n, ...) array with
-                for each voxel either a scalar or a vector.
-
-        Returns:
-            dict: same set of parameters but then possibly updated with a rotation.
-        """
-        sorted_eigenvalues, sorted_eigenvectors, ranking = DTIMeasures._sort_eigensystem(parameters_dict)
-        theta, phi, psi = tensor_cartesian_to_spherical(sorted_eigenvectors[0], sorted_eigenvectors[1])
-        return {'d': sorted_eigenvalues[:, 0], 'dperp0': sorted_eigenvalues[:, 1], 'dperp1': sorted_eigenvalues[:, 2],
-                'theta': theta, 'phi': phi, 'psi': psi}
-
     @staticmethod
     def fractional_anisotropy(d, dperp0, dperp1):
         """Calculate the fractional anisotropy (FA).
@@ -189,7 +97,7 @@ class DTIMeasures(object):
 
         if len(d.shape) > 1 and d.shape[1] > 1:
             fa = np.zeros(d.shape[:2])
-            for batch_start, batch_end in split_in_batches(d.shape[1], 100):
+            for batch_start, batch_end in split_in_batches(d.shape[1], max_batch_size=100):
                 fa[:, batch_start:batch_end] = compute(
                     d[:, batch_start:batch_end],
                     dperp0[:, batch_start:batch_end],
@@ -216,13 +124,12 @@ class DTIMeasures(object):
             ndarray: the standard deviation of the fraction anisotropy using error propagation of the diffusivities.
         """
         gradient = DTIMeasures._get_fractional_anisotropy_gradient(d, dperp0, dperp1)
-        covars = DTIMeasures._get_diffusivities_covariance_matrix(d_std, dperp0_std, dperp1_std,
-                                                                  covariances=covariances)
-
-        fa_std = np.zeros((gradient.shape[0]))
-        for ind in range(gradient.shape[0]):
-            fa_std[ind] = np.sqrt(np.dot(np.dot(gradient[ind], covars[ind]), gradient[ind]))
-        return np.nan_to_num(fa_std)
+        covars = create_covariance_matrix(
+            d.shape[0],
+            {'d.std': d_std, 'dperp0.std': dperp0_std, 'dperp1.std': dperp1_std},
+            ['d', 'dperp0', 'dperp1'],
+            covariances)
+        return np.nan_to_num(np.sqrt(voxelwise_vector_matrix_vector_product(gradient, covars, gradient)))
 
     @staticmethod
     def _get_fractional_anisotropy_gradient(d, dperp0, dperp1):
@@ -249,7 +156,7 @@ class DTIMeasures(object):
 
         d, dperp0, dperp1 = (np.squeeze(el).astype(np.float64) for el in [d, dperp0, dperp1])
 
-        return np.stack([
+        gradient = np.stack([
             (d ** 2 * (dperp0 + dperp1) + 2 * d * dperp0 * dperp1 - dperp0 ** 3
              - dperp0 ** 2 * dperp1 - dperp0 * dperp1 ** 2 - dperp1 ** 3)
             / (2 * (d ** 2 + dperp0 ** 2 + dperp1 ** 2) ** (3 / 2.)
@@ -264,29 +171,9 @@ class DTIMeasures(object):
                * np.sqrt(d ** 2 - d * (dperp0 + dperp1) + dperp0 ** 2 - dperp0 * dperp1 + dperp1 ** 2))
         ], axis=-1)
 
-    @staticmethod
-    def _get_diffusivities_covariance_matrix(d_std, dperp0_std, dperp1_std, covariances=None):
-        """Get the covariance matrix of the diffusivities.
-
-        This is required for the error propagation of the Fractional Anisotropy.
-        """
-        d_std, dperp0_std, dperp1_std = (np.squeeze(el) for el in [d_std, dperp0_std, dperp1_std])
-
-        covars = np.zeros((d_std.shape[0], 3, 3)).astype(np.float64)
-        covars[:, 0, 0] = d_std
-        covars[:, 1, 1] = dperp0_std
-        covars[:, 2, 2] = dperp1_std
-
-        covars **= 2
-
-        if covariances is not None:
-            covars[:, 0, 1] = covars[:, 1, 0] = np.squeeze(
-                covariances.get('d_to_dperp0', covariances.get('dperp0_to_d', 0)))
-            covars[:, 0, 2] = covars[:, 2, 0] = np.squeeze(
-                covariances.get('d_to_dperp1', covariances.get('dperp1_to_d', 0)))
-            covars[:, 1, 2] = covars[:, 2, 1] = np.squeeze(
-                covariances.get('dperp0_to_dperp1', covariances.get('dperp1_to_dperp0', 0)))
-        return covars
+        if len(gradient.shape) < 2:
+            return gradient[None, :]
+        return gradient
 
     @staticmethod
     def _sort_eigensystem(parameters_dict):
@@ -317,7 +204,7 @@ class DTIMeasures(object):
         return sorted_eigenvalues, sorted_eigenvectors, ranking
 
 
-class DKIMeasures(object):
+class DKIMeasures:
 
     @staticmethod
     def extra_optimization_maps(parameters_dict):
@@ -347,15 +234,15 @@ class DKIMeasures(object):
 
         nmr_voxels = parameters.shape[0]
         kernel_data = {'parameters': Array(parameters, ctype='mot_float_type'),
-                       'directions': Array(DKIMeasures._get_spherical_samples(), ctype='mot_float_type4',
-                                           offset_str='0'),
+                       'directions': Array(DKIMeasures._get_spherical_samples(), ctype='float4',
+                                           parallelize_over_first_dimension=False),
                        'nmr_directions': Scalar(DKIMeasures._get_spherical_samples().shape[0]),
                        'nmr_radial_directions': Scalar(256),
-                       'mks': Zeros((nmr_voxels,), ctype='mot_float_type'),
-                       'aks': Zeros((nmr_voxels,), ctype='mot_float_type'),
-                       'rks': Zeros((nmr_voxels,), ctype='mot_float_type')}
+                       'mks': Zeros((nmr_voxels,), ctype='float'),
+                       'aks': Zeros((nmr_voxels,), ctype='float'),
+                       'rks': Zeros((nmr_voxels,), ctype='float')}
 
-        DKIMeasures._get_compute_function(param_names).evaluate({'data': kernel_data}, nmr_instances=nmr_voxels)
+        DKIMeasures._get_compute_function(param_names).evaluate(kernel_data, nmr_voxels)
 
         return {'MK': kernel_data['mks'].get_data(),
                 'AK': kernel_data['aks'].get_data(),
@@ -364,25 +251,25 @@ class DKIMeasures(object):
     @staticmethod
     def _get_compute_function(param_names):
         def get_param_cl_ref(param_name):
-            return 'data->parameters[{}]'.format(param_names.index(param_name))
+            return 'parameters[{}]'.format(param_names.index(param_name))
 
         param_expansions = ['mot_float_type {} = params[{}];'.format(name, ind) for ind, name in enumerate(param_names)]
 
         return parse_cl_function('''
             double apparent_kurtosis(
                     global mot_float_type* params,
-                    mot_float_type4 direction,
-                    mot_float_type4 vec0,
-                    mot_float_type4 vec1,
-                    mot_float_type4 vec2){
+                    float4 direction,
+                    float4 vec0,
+                    float4 vec1,
+                    float4 vec2){
 
                 ''' + '\n'.join(param_expansions) + '''
 
-                mot_float_type adc = d *      pown(dot(vec0, direction), 2) +
-                                     dperp0 * pown(dot(vec1, direction), 2) +
-                                     dperp1 * pown(dot(vec2, direction), 2);
+                double adc = d *      pown(dot(vec0, direction), 2) +
+                             dperp0 * pown(dot(vec1, direction), 2) +
+                             dperp1 * pown(dot(vec2, direction), 2);
 
-                mot_float_type tensor_md = (d + dperp0 + dperp1) / 3.0;
+                double tensor_md = (d + dperp0 + dperp1) / 3.0;
 
                 double kurtosis_sum = KurtosisMultiplication(
                     W_0000, W_1111, W_2222, W_1000, W_2000, W_1110,
@@ -391,16 +278,16 @@ class DKIMeasures(object):
 
                 return pown(tensor_md / adc, 2) * kurtosis_sum;
             }
-        
+
             void get_principal_and_perpendicular_eigenvector(
                     mot_float_type d,
                     mot_float_type dperp0,
                     mot_float_type dperp1,
-                    mot_float_type4* vec0,
-                    mot_float_type4* vec1,
-                    mot_float_type4* vec2,
-                    mot_float_type4** principal_vec,
-                    mot_float_type4** perpendicular_vec){
+                    float4* vec0,
+                    float4* vec1,
+                    float4* vec2,
+                    float4** principal_vec,
+                    float4** perpendicular_vec){
 
                 if(d >= dperp0 && d >= dperp1){
                     *principal_vec = vec0;
@@ -413,19 +300,25 @@ class DKIMeasures(object):
                 *principal_vec = vec2;
                 *perpendicular_vec = vec0;
             }
-        
-            void calculate_measures(mot_data_struct* data){
+
+            void calculate_measures(global mot_float_type* parameters,
+                                    global float4* directions,
+                                    uint nmr_directions,
+                                    uint nmr_radial_directions,
+                                    global float* mks,
+                                    global float* aks,
+                                    global float* rks){
                 int i, j;
 
-                mot_float_type4 vec0, vec1, vec2;
+                float4 vec0, vec1, vec2;
                 TensorSphericalToCartesian(
                     ''' + get_param_cl_ref('theta') + ''',
                     ''' + get_param_cl_ref('phi') + ''',
                     ''' + get_param_cl_ref('psi') + ''',
                     &vec0, &vec1, &vec2);
 
-                mot_float_type4* principal_vec;
-                mot_float_type4* perpendicular_vec;
+                float4* principal_vec;
+                float4* perpendicular_vec;
                 get_principal_and_perpendicular_eigenvector(
                     ''' + get_param_cl_ref('d') + ''',
                     ''' + get_param_cl_ref('dperp0') + ''',
@@ -435,28 +328,26 @@ class DKIMeasures(object):
 
                 // Mean Kurtosis integrated over a set of directions
                 double mean = 0;
-                for(i = 0; i < data->nmr_directions; i++){
-                    mean += (apparent_kurtosis(data->parameters, data->directions[i], vec0, vec1, vec2) - mean)
-                                / (i + 1);
+                for(i = 0; i < nmr_directions; i++){
+                    mean += apparent_kurtosis(parameters, directions[i], vec0, vec1, vec2);
                 }
-                *(data->mks) = clamp(mean, (double)0, (double)3);
+                *(mks) = clamp(mean / nmr_directions, 0.0, 3.0);
 
 
                 // Axial Kurtosis over the principal direction of diffusion
-                *(data->aks) = clamp(apparent_kurtosis(data->parameters, *principal_vec, vec0, vec1, vec2),
-                                   (double)0, (double)10);
+                *(aks) = clamp(apparent_kurtosis(parameters, *principal_vec, vec0, vec1, vec2), 0.0, 10.0);
 
 
                 // Radial Kurtosis integrated over a unit circle around the principal eigenvector.
                 mean = 0;
-                mot_float_type4 rotated_vec;
-                for(i = 0; i < data->nmr_radial_directions; i++){
+                float4 rotated_vec;
+                for(i = 0; i < nmr_radial_directions; i++){
                     rotated_vec = RotateOrthogonalVector(*principal_vec, *perpendicular_vec,
-                                                         i * (2 * M_PI_F) / data->nmr_radial_directions);
+                                                         i * (2 * M_PI_F) / nmr_radial_directions);
 
-                    mean += (apparent_kurtosis(data->parameters, rotated_vec, vec0, vec1, vec2) - mean) / (i + 1);
+                    mean += (apparent_kurtosis(parameters, rotated_vec, vec0, vec1, vec2) - mean) / (i + 1);
                 }
-                *(data->rks) = max(mean, (double)0);
+                *(rks) = max(mean, 0.0);
             }
         ''', dependencies=[get_component('library_functions', 'RotateOrthogonalVector')(),
                            get_component('library_functions', 'TensorSphericalToCartesian')(),
@@ -728,3 +619,93 @@ class DKIMeasures(object):
                          [-0.0166, -0.8421, -0.5391],
                          [0.7741, -0.2931, 0.5610],
                          [-0.9636, 0.0579, 0.2611]])
+
+
+class NODDIMeasures:
+
+    @staticmethod
+    def noddi_watson_extra_optimization_maps(results):
+        """Computes the NDI and ODI for the NODDI Watson model"""
+        return {'NDI': results['w_ic.w'] / (results['w_ic.w'] + results['w_ec.w']),
+                'ODI': np.arctan2(1.0, results['NODDI_IC.kappa']) * 2 / np.pi}
+
+    @staticmethod
+    def noddi_watson_extra_sampling_maps(results):
+        """Computes the NDI and ODI per sample and average over the derived values."""
+        ndi = results['w_ic.w'] / (results['w_ic.w'] + results['w_ec.w'])
+        odi = np.arctan2(1.0, results['NODDI_IC.kappa']) * 2 / np.pi
+
+        return {'NDI': np.mean(ndi, axis=1), 'NDI.std': np.std(ndi, axis=1),
+                'ODI': np.mean(odi, axis=1), 'ODI.std': np.std(odi, axis=1)}
+
+    @staticmethod
+    def noddi_bingham_extra_optimization_maps(results):
+        """Computes the ODI's and Dispersion Anisotropic Index (DAI) for the NODDI Bingham model"""
+        def compute(ind, kappa, beta):
+            return {'ODI_p{}'.format(ind): np.arctan2(1.0, kappa - beta) * 2 / np.pi,
+                    'ODI_s{}'.format(ind): np.arctan2(1.0, kappa) * 2 / np.pi,
+                    'ODI{}'.format(ind): np.arctan2(1.0, np.sqrt(np.abs(kappa * (kappa - beta)))) * 2 / np.pi,
+                    'DAI{}'.format(ind): np.arctan2(beta, kappa - beta) * 2 / np.pi}
+
+        output = {}
+        for ind in range(2):
+            if 'BinghamNODDI_IN{}.k1'.format(ind) in results:
+                output.update(compute(
+                    ind,
+                    results['BinghamNODDI_IN{}.k1'.format(ind)],
+                    results['BinghamNODDI_IN{}.k1'.format(ind)] / results['BinghamNODDI_IN{}.kw'.format(ind)]))
+        return output
+
+    @staticmethod
+    def noddi_bingham_extra_sampling_maps(results):
+        """Computes the ODI's and Dispersion Anisotropic Index (DAI) for the NODDI Bingham model.
+
+        This computes the indices per sample and takes the mean and std. over that.
+        """
+        def compute(ind, kappa, beta):
+            odi_p = np.arctan2(1.0, kappa - beta) * 2 / np.pi
+            odi_s = np.arctan2(1.0, kappa) * 2 / np.pi
+            odi = np.arctan2(1.0, np.sqrt(np.abs(kappa * (kappa - beta)))) * 2 / np.pi
+            dai = np.arctan2(beta, kappa - beta) * 2 / np.pi
+
+            return {'ODI_p{}'.format(ind): np.mean(odi_p, axis=1),
+                    'ODI_p{}.std'.format(ind): np.std(odi_p, axis=1),
+                    'ODI_s{}'.format(ind): np.mean(odi_s, axis=1),
+                    'ODI_s{}.std'.format(ind): np.std(odi_s, axis=1),
+                    'ODI{}'.format(ind): np.mean(odi, axis=1),
+                    'ODI{}.std'.format(ind): np.std(odi, axis=1),
+                    'DAI{}'.format(ind): np.mean(dai, axis=1),
+                    'DAI{}.std'.format(ind): np.std(dai, axis=1)}
+
+        output = {}
+        for ind in range(2):
+            if 'BinghamNODDI_IN{}.k1'.format(ind) in results:
+                output.update(compute(
+                    ind,
+                    results['BinghamNODDI_IN{}.k1'.format(ind)],
+                    results['BinghamNODDI_IN{}.k1'.format(ind)] / results['BinghamNODDI_IN{}.kw'.format(ind)]))
+        return output
+
+
+def noddi_dti_maps(results):
+    """Compute NODDI-like statistics from Tensor/Kurtosis parameter fits.
+
+    This uses :func:`mdt.utils.compute_noddi_dti` for the computation, see there for more information.
+
+    Args:
+        results (mdt.models.composite.ExtraOptimizationMapsInfo): the results data, should contain at least:
+
+            - d (ndarray): principal diffusivity
+            - dperp0 (ndarray): primary perpendicular diffusion
+            - dperp1 (ndarray): primary perpendicular diffusion
+
+            And, if present, we also use these:
+
+            - FA (ndarray): if computed already, the Fractional Anisotropy of the given diffusivities
+            - MD (ndarray): if computed already, the Mean Diffusivity of the given diffusivities
+            - MK (ndarray): if computing for Kurtosis, the computed Mean Kurtosis. If not given, we assume unity.
+
+    Returns:
+        dict: maps for the the NODDI-DTI, NDI and ODI measures.
+    """
+    return compute_noddi_dti(results.model, results.input_data, results.results, noddi_d=1.7e-9)
