@@ -2,4 +2,4 @@ __author__ = 'Robbert Harms'
 __date__ = "2014-11-27"
 __license__ = "LGPL v3"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
