@@ -2,10 +2,10 @@ __author__ = 'Robbert Harms'
 __date__ = "2014-06-19"
 __license__ = "LGPL v3"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
-class AbstractParameterDependency(object):
+class AbstractParameterDependency:
 
     @property
     def pre_transform_code(self):
