@@ -1,13 +1,11 @@
-from mot.lib.cl_function import SimpleCLFunctionParameter
-from .parameters import FreeParameter
+from .parameters import FreeParameter, CurrentModelSignalParam
 from .model_functions import SimpleModelCLFunction
-from .parameter_functions.transformations import CosSqrClampTransform
 
 __author__ = 'Robbert Harms'
 __date__ = "2014-08-05"
 __license__ = "LGPL v3"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class SignalNoiseModel(SimpleModelCLFunction):
@@ -42,7 +40,7 @@ class JohnsonNoise(SignalNoiseModel):
 
         """
         super().__init__(
-            'double', 'JohnsonNoise', 'JohnsonNoise',
-            [SimpleCLFunctionParameter('double', 'signal'),
-             FreeParameter('mot_float_type', 'eta', False, 0.1, 0, 100, parameter_transform=CosSqrClampTransform())],
+            'double', 'JohnsonNoise', [
+                CurrentModelSignalParam('signal'),
+                FreeParameter('mot_float_type eta', False, 0.1, 0, 1e5)],
             'return hypot(signal, (double)eta);')
