@@ -13,7 +13,7 @@ __author__ = 'Robbert Harms'
 __date__ = "2014-02-06"
 __license__ = "LGPL v3"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class Protocol(collections.Mapping):
@@ -40,6 +40,7 @@ class Protocol(collections.Mapping):
         self._columns = {}
         self._preferred_column_order = ('gx', 'gy', 'gz', 'G', 'Delta', 'delta', 'TE', 'T1', 'b', 'q', 'maxG')
         self._virtual_columns = [VirtualColumnB(),
+                                 VirtualColumn_g_spherical(),
                                  SimpleVirtualColumn('Delta', lambda protocol: get_sequence_timings(protocol)['Delta']),
                                  SimpleVirtualColumn('delta', lambda protocol: get_sequence_timings(protocol)['delta']),
                                  SimpleVirtualColumn('G', lambda protocol: get_sequence_timings(protocol)['G'])]
@@ -337,7 +338,7 @@ class Protocol(collections.Mapping):
                 clustered_shells.append(new_shell)
             return clustered_shells
 
-        clusters = cluster_b_values(np.sort(self.get_column('b')[self.get_weighted_indices()]))
+        clusters = cluster_b_values(np.sort(np.squeeze(self.get_column('b')[self.get_weighted_indices()])))
 
         cluster_info = []
         for cluster in clusters:
@@ -489,7 +490,10 @@ class Protocol(collections.Mapping):
         """
         for virtual_column in self._virtual_columns:
             if virtual_column.name == column_name:
-                return np.reshape(virtual_column.get_values(self), (-1, 1))
+                data = virtual_column.get_values(self)
+                if data.ndim == 1:
+                    return np.reshape(data, (-1, 1))
+                return data
 
         raise KeyError('The given column name "{}" could not be found in this protocol.'.format(column_name))
 
@@ -534,7 +538,7 @@ class Protocol(collections.Mapping):
         return s
 
 
-class VirtualColumn(object):
+class VirtualColumn:
 
     def __init__(self, name):
         """The interface for generating virtual columns.
@@ -590,6 +594,17 @@ class VirtualColumnB(VirtualColumn):
                                    (sequence_timings['Delta'] - (sequence_timings['delta'] / 3))), (-1, 1))
 
 
+class VirtualColumn_g_spherical(VirtualColumn):
+
+    def __init__(self):
+        super().__init__('g_sph')
+
+    def get_values(self, parent_protocol):
+        from mdt.utils import cartesian_to_spherical
+        thetas, phis = cartesian_to_spherical(parent_protocol['g'], ensure_right_hemisphere=False)
+        return np.array([thetas, phis]).T
+
+
 def get_sequence_timings(protocol):
     """Return G, Delta and delta, estimate them if necessary.
 
