@@ -5,7 +5,7 @@ __author__ = 'Robbert Harms'
 __date__ = "2014-02-14"
 __license__ = "LGPL v3"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
+__email__ = "robbert@xkls.nl"
 
 
 class MyColourBarTickLocator(LinearLocator):
