@@ -2,4 +2,4 @@ __author__ = 'Robbert Harms'
 __date__ = "2014-05-21"
 __license__ = "LGPL v3"
 __maintainer__ = "Robbert Harms"
-__email__ = "robbert.harms@maastrichtuniversity.nl"
\ No newline at end of file
+__email__ = "robbert@xkls.nl"
